/*
 *  svinnode.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  svinnode.h   */
/*  Copyright 1990 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef SVINNODE_DOT_H
#define SVINNODE_DOT_H
#include "hrdarth.h"

#include "portable.h"
#include "entenm.h"
#include "debug.h"
#include "dfnode.h"
#include "signode.h"

class InputNode: public SignalStr {
#line 60 "../svinnode.usr"
 
	void FileInit() ;
	void BadFile() ;
	virtual TimingDescription * GetInitTiming(int Channel);
	int Channels ;
	int32 * ElementSize ;
	int32 GetElementSize(int channel) {return ElementSize[channel];}
	class InputFile * TheInputFile ;
	double * SampleRateRatio ;
	double * CumulativeSamples ;
#line 22 "svinnode.h"
	const char *  FileName_1;
public:
	InputNode (const char * Name, const char * FileName);
	virtual ~InputNode();
	const char * GetFileName() const {return FileName_1;}
	virtual int CheckSafeDelete() ;
	virtual ErrCode DoNode(int32);
	void Describe(OutTokens& Out, ListEntity Option);
	virtual int CppList(OutTokens& Out, CppListCmds Cmd);
	void DisplayHeader ();
	void IgnoreHeaderCount ();
};

#endif /* #ifdef SVINNODE_DOT_H */
