#!/bin/sh
#  do_check_index.sh from ObjectProDSP 0.2
#  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
#  Licensed for free use and distribution under version 2 of the Gnu General
#  Public License. Please see file COPYING for details and restrictions.
#  
#  ObjectProDSP is a trademark of Mountain Math Software.
#  
if [ -e ovr_app.idx ]  ;then
	if [ -e ovr_front.idx ]  ;then
		if [ -e over_all.idx ] ;then 
			 mv over_all.idx old_over_all.idx
		fi
		cp ovr_front.idx over_all.idx 
		cat ovr_app.idx >> over_all.idx 
		$1 over_all
	fi
fi

