/*
 *  gui.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef GUI_DOT_H
#define GUI_DOT_H
#include "xgui.h"
// dummy until we get rid of all CGI stuff that might include gui.h
#endif /* #ifdef GUI_DOT_H */
