/*
 *  menuprsy.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  menuprsy.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#include "portable.h"
#define YTAB_DOT_H
#include "menuprsl.h"
#include "stdyacp.h"
#include "domenus.h"
#include "menustr.h"
#include "parammen.h"
#include "yacoper.h"
#include "objinit.h"
#ifdef __NT_VC__
#include <malloc.h>
#endif


Value * AddToWordString(Value *,const char * ,int = 1);
Value * CreateWordString(const char * );
Value * ConcatWordString(Value *, int );

extern char * the_help_default_file ;

void FoundHelpDefaultFile(const char *);

const char * GetNodeNameFromAction(const char * TheName) ;
const char * GetNodeName(const char * TheName, const char * Prefix,
	const char * Suffix) ;
int SkipAnItem(MenuItem * AnItem, const char * MenuName) ;
void FoundClassRelation(const char *, const char *);
void ClassHasMembers(const char * Name) ;

#ifdef SCO_NO_ALLOCA
#include <malloc.h>
#define alloca malloc
#endif
