/*
 *  strtlex.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
# define TEXT_MODE 2
#define BEGIN yybgin = yysvec + 1 +
/*
 * extern struct yysvf yysvec[], *yybgin;
 * struct yysvf { 
 *	struct yywork *yystoff;
 *	struct yysvf *yyother;
 *	int *yystops;};
 */

