#include "ObjProNet/dfnode.h"

#include "ObjProUsr/blockplt.h"
#include "ObjProGen/outtok.h"
#include "ObjProGui/intfc.h"
#include "ObjProGui/remmen.h"
#include "ObjProGui/user.h"
#include "ObjProGui/dynmnu.h"
#include "ObjProArith/typout.h"
#include "ObjProGui/array.h"
#include "ObjProGui/interinit.h"
#include "ObjProGen/stattyp.h"
static EntityList * GenericBlockPlotNodeList = 0 ;
static InteractiveEntity * IntEntGenericBlockPlot ;
void GenericBlockPlotNodesInit();
GenericBlockPlot::GenericBlockPlot (const char * Name, int32 procEltSz, 
		const char * procCaption, double procMinY, double procMaxY, 
		double procMinX, double procMaxX, int16 procNumberBlocks, 
		int16 procScaleFlag, int32 procBlockSize, 
		TimingType procTheTimingType):
	GenericPlot(Name, 1, procEltSz, procCaption, procMinY, procMaxY, 
	procMinX, procMaxX, procNumberBlocks, procScaleFlag, procBlockSize, 
	procTheTimingType, 0.000000e+00, PlotYs)
,
	procEltSz_1(procEltSz),
	procCaption_2(procCaption),
	procMinY_3(procMinY),
	procMaxY_4(procMaxY),
	procMinX_5(procMinX),
	procMaxX_6(procMaxX),
	procNumberBlocks_7(procNumberBlocks),
	procScaleFlag_8(procScaleFlag),
	procBlockSize_9(procBlockSize),
	procTheTimingType_10(procTheTimingType)
{
} // end constructor

GenericBlockPlot::~GenericBlockPlot()
{
} // end destructor

int GenericBlockPlot::CheckSafeDelete()
{
	int Safe_Check_Return = DfNode::CheckSafeDelete();
	if (!Safe_Check_Return) return 0;
	return 1;
} // end check safe delete

void GenericBlockPlotNodesInit()
{
	if (GenericBlockPlotNodeList)  return ;
	GenericBlockPlotNodeList = new EntityList;
	IntEntGenericBlockPlot = new InteractiveEntity("GenericBlockPlot", GenericBlockPlotNodeList,
		0, InteractiveNode, "blockplt.h",
		0, "GenericPlot");
	TheNodes->Append(IntEntGenericBlockPlot);
} // end initalization

static InitObj LocalInit(GenericBlockPlotNodesInit, "GenericBlockPlot", "GenericPlot");

