/*
 *  menuspc.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef MENUSPC_DOT_H
#define MENUSPC_DOT_H
/*  menuspc.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#include "menu.h"
#include "mentyp.h"

class MenuLine ;
class Menu ;

struct GenericOptions {
	MenuLine * Lines ;
	DoMenuReturn * Actions ;
	int NumActions ;
} ;
const UseMainMenu = 0 ;
DoMenuReturn MenuDriver(Menu&, const char* , GenericOptions&,
	const char *Input=0) ;
extern struct GenericOptions GenericMain ;
DoMenuReturn DoMenuItem(Menu& ThisMenu, const char * GetHere, int limit,
	GenericOptions& Gen, const char * Input) ;

#endif /* #ifdef MENUSPC_DOT_H */
