/*  dyplotc.h   */
/*  Copyright 1990 Mountain Math Software  */
/*  All Rights Reserved                    */

#include "ObjProArith/hrdarth.h"
#include "ObjProUsr/genplot.h"

#include <complex.h>
#include "ObjProArith/hrdarth.h"
#include "ObjProGui/sigintfc.h"
#include "ObjProComGui/cgidbg.h"
#include "ObjProDSP/pltenm.h"
#include "ObjProGui/yacintfc.h"
#include "ObjProUsr/dyplot.h"


DfNode * Plot::target_replacement(const char * target)
{
	return default_output_replacement(target);
}

int Plot::InitAfterLinked()
{
	return set_read_binary_flag(binary_read_flag) ;
}

void Plot::CheckSetup()
{
	// LogOut << "Plot::CheckSetup before DynamicInit\n" ;
	DynamicInit();
	// LogOut << "Plot::CheckSetup before CheckRemoteSetup\n" ;
	CheckRemoteSetup();
	// LogOut << "PlotId = " << GetPlotId() << "\n" ;
	// LogOut << "Plot::CheckSetup before Exit\n" ;
}

