/*
 *  miscel.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef MISCEL_DOT_H
#define MISCEL_DOT_H
#include "ObjProGui/basic.h"
class Miscellaneous :public UserEntity {
public:
	Miscellaneous(const char * Name):UserEntity(Name){}
};
#endif /* #ifdef MISCEL_DOT_H */
