/*
 *  feedbck.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef FEEDBCK_DOT_H
#define FEEDBCK_DOT_H
/*  feedbck.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */


struct LoopMember {
	class DfNodeInLink * In ;
	class DfNodeOutLink * Out ;
	LoopMember(DfNodeInLink *in, DfNodeOutLink * out) {In=in; Out=out;}
};


struct FeedbackLoop {
	class LoopList * AllLoops ;
	FeedbackLoop() ;
	~FeedbackLoop() ;
	void AppendNode(class DfNode * TheNode) ;
	void AppendNode(DfNodeInLink *In, DfNodeOutLink *Out) ;
	void AddLoop() ;
	int CheckEndFeedback(DfNodeOutLink * CheckEnd) ;
	void Merge(FeedbackLoop * ToMerge);
	void Dump();
} ;

#endif /* #ifdef FEEDBCK_DOT_H */
