/*
 *  examp.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  examp.h   */
/*  Copyright 1991 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef EXAMP_DOT_H
#define EXAMP_DOT_H

class DirectoriesDspPP {
public:
	const char * const Root ;
	const char * const Examples ;
	const char * const HelpFiles ;
	const char * const Home ;
	DirectoriesDspPP();
};

class ExamplesDspPP {
public:
	ExamplesDspPP();
	void DescribeExample(const char * Examp) const ;
	void ExecuteExample(const char * Examp, const char * Over) const ;
	void ReadExample(const char * Examp) const ;
};

extern DirectoriesDspPP * TheDirectories ;
extern ExamplesDspPP * TheExamples ;


#endif /* #ifdef EXAMP_DOT_H */
