/*
 *  lexhead.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  lexhead.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef LEXHEAD_DOT_H
#define LEXHEAD_DOT_H
#include <stream.h>

#include "portable.h"
#include "prsdrv.h"

#include "y.tab.h"

ValueType * CheckDeclarator(const char *) ;
ValueType * CheckDeclared(const char *) ;
extern int32 OctIntConvert(char *) ;
extern int CommandCheck(char *) ;
extern int FoundName(char *) ;
extern void NetEditCommand();


extern int Getc() ; /* interface to stream input */

const char * CondGetName(const char * Prompt, const char * IfError,
	const char * Err1 = 0, const char * Err2 = 0, const char * Err3 = 0,
	const char * Err4 = 0, const char * Err5 = 0);

const char * CondGetRawBufLine(const char * Prompt, const char * IfError,
	const char * Err1 = 0, const char * Err2 = 0);


#endif /* #ifdef LEXHEAD_DOT_H */
