/*
 *  flt.c from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#include <stdio.h>
#include <ctype.h>
#include "ObjProGen/cpyrght_exe.h"

void CheckOut(c)
int c;
{
	int is_good ;
	is_good = isascii(c);
	if (is_good) if (!isprint(c)) if (c != '\n')  is_good = 0 ;
	if (is_good) {
		putchar(c) ;
		return ;
	}
	fprintf(stderr,"Bad character in ouput 0x%x\n", c);
}


main()
{
	int ch ;
	while (ch = getchar()) if (ch < 0) break ;
		else if (ch == '\b') getchar() ;
		else if (ch == '_') {
			int ch2;
			ch2 = getchar() ;
			if(ch2 == '\b') continue ;
			if(ch2 == '_') {
				int ch3 ;
				ch3 = getchar();
				if (ch3 == '\b') {
					CheckOut(' ') ;
					continue ;
				}
				CheckOut(ch);
				if (ch2 < 0) break ;
				CheckOut(ch2);
				if (ch3 < 0) break ;
				CheckOut(ch3);
				continue ;
			}
			CheckOut(ch);
			if (ch2 < 0) break ;
			CheckOut(ch2);
			continue;
		} else CheckOut(ch);
}
