/*  sigintfc.c   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */

// Interface to signal generator and display nodes
class InteractiveEntityList * TheSignals ;
class InteractiveEntityList * TheDisplays ;

