/*
 *  cmpseq.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  cmpseq.h   */
/*  Copyright 1991 Mountain Math Software  */
/*  All Rights Reserved                    */
class BufferStatus ;
class DfNodeOutLink ;

class ExactBufferSize {
	int32 ProviderIncrement ;
	int32 ConsumerIncrement ;
	int32 OverlapWords ;
	int32 LeftOver ;
	int32 NetworkCycleWords ;
public:
	ExactBufferSize(DfNodeOutLink * ProviderLink, int BufferOutChannel);
	int32 ComputeSize(BufferStatus& Constraints);
	int ComputeSequence(int32 BufferSize, int ** ReturnSequence=0) ;
};
	

int CountNeeded(int32 Dividend, int32 Divisor) ;

