/*  topol.c   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#include "debug.h"
#include "topol.h"

int GlobalInCount = 0 ;
// used for TIC30 debugging

void SourceData::GenData (int32, void *) 
{
	DbgError("SourceData::GenData", "dummy virtual function called");
}

