/*
 *  menuprsl.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  menuprsl.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef MENUPRSL_DOT_H
#define MENUPRSL_DOT_H
#include <string.h>
#ifdef __NT_VC__
#include <io.h>
#endif
#include "portable.h"
#include "stdyac.h"
#include "text.h"
#include "lexnum.h"
#include "input.h"
struct Double * MakeDouble(const char *) ;
char * MakeOperatorCommand(const char * Base,const char * OperatorString);
#ifndef YTAB_DOT_H
#include "y.tab.h"
void FoundInt(int);
void PushFile(const char * );
void GetEmbeddedName(const char *);
extern int EmbSt ;
#endif /* #ifdef YTAB_DOT_H */

#endif /* #ifdef MENUPRSL_DOT_H */
