#include "ObjProNet/dfnode.h"

#include "ObjProUsr/genplot.h"
#include "ObjProGen/outtok.h"
#include "ObjProGui/intfc.h"
#include "ObjProGui/remmen.h"
#include "ObjProGui/user.h"
#include "ObjProGui/dynmnu.h"
#include "ObjProArith/typout.h"
#include "ObjProGui/array.h"
#include "ObjProGui/interinit.h"
#include "ObjProGen/stattyp.h"
#include "ObjProUsr/gpltstr.h"
static EntityList * GenericPlotNodeList = 0 ;
static InteractiveEntity * IntEntGenericPlot ;
void GenericPlotNodesInit();
GenericPlot::GenericPlot (const char * Name, int16 procIn, int32 procEltSz, 
		const char * procCaption, double procMin, double procMax, 
		double procMinimumX, double procMaximumX, 
		int16 procNumberBlocks, int16 procScaleFlag, 
		int32 procBlockSize, TimingType procTheTimingType, 
		double procXYSamplesPerPlot, 
		PlottingStreamType procStreamType):
	GenericPlotStr(Name, procIn, 
	 new StreamStr(procEltSz , procBlockSize), procCaption, procMin, procMax, procMinimumX, procMaximumX, 
	procNumberBlocks, procScaleFlag, procTheTimingType, PlotDynStatic, 
	procXYSamplesPerPlot, procStreamType)
,
	procIn_1(procIn),
	procEltSz_2(procEltSz),
	procCaption_3(procCaption),
	procMin_4(procMin),
	procMax_5(procMax),
	procMinimumX_6(procMinimumX),
	procMaximumX_7(procMaximumX),
	procNumberBlocks_8(procNumberBlocks),
	procScaleFlag_9(procScaleFlag),
	procBlockSize_10(procBlockSize),
	procTheTimingType_11(procTheTimingType),
	procXYSamplesPerPlot_12(procXYSamplesPerPlot),
	procStreamType_13(procStreamType)
{
} // end constructor

GenericPlot::~GenericPlot()
{
} // end destructor

int GenericPlot::CheckSafeDelete()
{
	int Safe_Check_Return = DfNode::CheckSafeDelete();
	if (!Safe_Check_Return) return 0;
	return 1;
} // end check safe delete

void GenericPlotNodesInit()
{
	if (GenericPlotNodeList)  return ;
	GenericPlotNodeList = new EntityList;
	IntEntGenericPlot = new InteractiveEntity("GenericPlot", GenericPlotNodeList,
		0, InteractiveNode, "genplot.h",
		0, "GenericPlotStr");
	TheNodes->Append(IntEntGenericPlot);
} // end initalization

static InitObj LocalInit(GenericPlotNodesInit, "GenericPlot", "GenericPlotStr");

