#include "shrbuf.h"
#include <stream.h>

SharedBuffer TheSharedBuffer("TheSharedBuffer");

void SharedBuffer::declare_name(ostream& out)
{
	out << "\nextern MachWordType " << TheSharedBuffer.Name << "[];\n\n" ;
}

void SharedBuffer::declare_space(ostream& out)
{
	out << "\nMachWordType " << TheSharedBuffer.Name << "[" <<
        TheSharedBuffer.GetMaxSize() << "];\n\n" ;
}
