// This file contains dummy versions of all timinfo.h routines
// it is linked when timing analysis is not being performed

#include "timinfo.h"

int TimingCheckOn = 0 ;
int TimingEntrysLimit = 0 ;


TimingInfo::TimingInfo()
{}


