/*
 *  typout.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef TYPOUT_DOT_H
#define TYPOUT_DOT_H
/*  typout.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#include "ObjProArith/hrdarth.h"
const char * TypeToString(MachWord x);
const char * TypeToString(AccMachWord x);
const char * TypeToString(CxMachWord& x);
const char * TypeToString(CxAccMachWord& x);
const char * TypeToString(int32 x);
const char * TypeToString(double x);
const char * TypeToString(class complex& x);

typedef int16 CastMachWord ;
typedef int32 CastAccMachWord ;

#endif /* #ifdef TYPOUT_DOT_H */
