/*
 *  mknodelx.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  mknodelx.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#include <stream.h>
#include <fstream.h>
#include <stdio.h>
#include "lexnum.h"
#include "y.tab.h"
#include "stdyac.h"
#include "portable.h"
#include "text.h"

void EmbeddedLine() ;
void FoundInt(int32);
struct Double * MakeDouble(const char *);
enum NewStateOption {BeginNull=0, BeginCpp, BeginParamValue, BeginTimeParam,
 	BeginMenuName, BeginEmbedded};

struct LexReturn {
	int Terminal ;
	NewStateOption NewState ;
	LexReturn(int term, NewStateOption state):Terminal(term),
		NewState(state){}
};

int GetIncludeType(const char * yytext);
LexReturn& FoundName(const char * Text, YYSTYPE& yylval);
const char * TerminalToCppType(int terminal) ;
const char * TerminalToCType(int terminal) ;
const char * TerminalToString(int terminal) ;

extern int last_lex_state ;

