/*
 *  stattyp.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
void OutStringStaticValue(class ostream& Out, const char * TypeName,
	void * Value) ;
const char * GetStatInitName(const char * TyoeName) ;
int GetStaticElementSize(const char * TypeName) ;
const char * FullFloatValue(double x) ;
const char * FullDoubleValue(double x) ;

