/*  warn.C   */
/*  Copyright 1991 Mountain Math Software  */
/*  All Rights Reserved                    */
#include <stream.h>
#include "stdyac.h"
int WarningCount = 0 ;
void GiveWarning(const char * Msg)
{
	cerr << "Warning: " << Msg << " at line " << LineNumber << ".\n" ;
	WarningCount++ ;
}


