#include "ObjProNet/dfnode.h"

#include "ObjProUsr/gpltstr.h"
#include "ObjProGen/outtok.h"
#include "ObjProGui/intfc.h"
#include "ObjProGui/remmen.h"
#include "ObjProGui/user.h"
#include "ObjProGui/dynmnu.h"
#include "ObjProArith/typout.h"
#include "ObjProGui/array.h"
#include "ObjProGui/interinit.h"
#include "ObjProGen/stattyp.h"
#include "ObjProArith/hrdcmp.h"
static EntityList * GenericPlotStrNodeList = 0 ;
static InteractiveEntity * IntEntGenericPlotStr ;
void GenericPlotStrNodesInit();
GenericPlotStr::GenericPlotStr (const char * Name, int16 In, StreamStr*StreamIn, 
		const char * Caption, double Min, double Max, 
		double MinimumX, double MaximumX, int16 NumberBlocks, 
		int16 ScaleFlag, TimingType TheTimingType, 
		PlotDynType DynamicType, double XYSamplesPerPlot, 
		PlottingStreamType StreamType):
	PlotNode(Name, In, StreamIn, ScaleFlag, TheTimingType)
,
	In_1(In),
	StreamIn_2(StreamIn),
	Caption_3(Caption),
	Min_4(Min),
	Max_5(Max),
	MinimumX_6(MinimumX),
	MaximumX_7(MaximumX),
	NumberBlocks_8(NumberBlocks),
	ScaleFlag_9(ScaleFlag),
	TheTimingType_10(TheTimingType),
	DynamicType_11(DynamicType),
	XYSamplesPerPlot_12(XYSamplesPerPlot),
	StreamType_13(StreamType)
{
#line 54 "../gpltstr.usr"
 
	// XScale = 0 ;
	// YScale = 0 ;
	PlotCaption = 0 ;
	switch (DynamicType) {
case PlotDynStatic:
		DynamicInit();
		break ;
case PlotDynDyn:
		break ;
	}
#line 52 "../gpltstr.cxx"
} // end constructor

GenericPlotStr::~GenericPlotStr()
{
} // end destructor

int GenericPlotStr::CheckSafeDelete()
{
	int Safe_Check_Return = DfNode::CheckSafeDelete();
	if (!Safe_Check_Return) return 0;
	return 1;
} // end check safe delete

void GenericPlotStrNodesInit()
{
	if (GenericPlotStrNodeList)  return ;
	GenericPlotStrNodeList = new EntityList;
	IntEntGenericPlotStr = new InteractiveEntity("GenericPlotStr", GenericPlotStrNodeList,
		0, InteractiveNode, "gpltstr.h",
		0, "PlotNode");
	TheNodes->Append(IntEntGenericPlotStr);
} // end initalization

static InitObj LocalInit(GenericPlotStrNodesInit, "GenericPlotStr", "PlotNode");

