
/*  A Bison parser, made from ../menu_b.y with Bison version GNU Bison version 1.22
  */

#define YYBISON 1  /* Identify Bison output.  */

#define	HELP_DEF	258
#define	DEF_HELP_FILE	259
#define	CONFLICT	260
#define	WAIT	261
#define	DEFINED_MENU	262
#define	DEFINED_ACTION	263
#define	MENU_ADDITION	264
#define	ITEM_REF	265
#define	DEFAULT	266
#define	ITEM_HELP	267
#define	MENU_LOCAL	268
#define	MENU_REMOTE	269
#define	MENU_REMOTE_OPTIONS	270
#define	MAINS	271
#define	INIT	272
#define	INCLUDE_LOCAL	273
#define	INCLUDE_REMOTE	274
#define	MULTIPLE	275
#define	USE	276
#define	DYNAMIC	277
#define	HISTORY	278
#define	SELECT	279
#define	ADD_TO_MENU	280
#define	TEMPLATE	281
#define	ORPHAN	282
#define	SHIFT_R	283
#define	SHIFT_L	284
#define	LEQ	285
#define	GEQ	286
#define	EQU	287
#define	NEQ	288
#define	AND	289
#define	OR	290
#define	OPERATOR	291
#define	NAME	292
#define	STRING	293
#define	LOCAL	294
#define	REMOTE	295
#define	LOCAL_REMOTE	296
#define	REMOTE_OPTIONS	297
#define	MENU	298
#define	NEW_MENU	299
#define	HELP_FILE	300
#define	DYNAMIC_MENU	301
#define	TEXT	302
#define	INTEGER_VAL	303
#define	DOUBLE_VAL	304
#define	USER_COPYRIGHT	305
#define	EXPLICIT_STRING	306

#line 40 "../menu_b.y"

#include <stream.h>
#include <stdio.h>
#include "menuprsy.h"

#line 45 "../menu_b.y"
typedef union {
	int32 Int ;
	const char * Name ;
	union Value * Val ;
	struct TextFragmentList * Text ;
	struct Double * Dbl ;
	class MenuType * TheMenuType ;
	struct Compound * Tree ;
	} YYSTYPE;

#ifndef YYLTYPE
typedef
  struct yyltype
    {
      int timestamp;
      int first_line;
      int first_column;
      int last_line;
      int last_column;
      char *text;
   }
  yyltype;

#define YYLTYPE yyltype
#endif

#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		214
#define	YYFLAG		-32768
#define	YYNTBASE	75

#define YYTRANSLATE(x) ((unsigned)(x) <= 306 ? yytranslate[x] : 122)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    73,     2,     2,    74,    63,    72,     2,    59,
    60,    71,    70,    57,    62,     2,    64,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,    61,    58,    65,
    55,    66,    69,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
    54,     2,    56,    67,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    52,    68,    53,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
    16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
    26,    27,    28,    29,    30,    31,    32,    33,    34,    35,
    36,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,    50,    51
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     3,     5,     8,    10,    11,    16,    18,    21,    27,
    32,    34,    38,    40,    43,    45,    47,    49,    51,    53,
    55,    57,    59,    61,    65,    70,    77,    82,    87,    89,
    91,    96,    99,   108,   110,   114,   116,   125,   131,   134,
   137,   141,   144,   147,   150,   151,   153,   154,   157,   160,
   161,   164,   167,   170,   174,   175,   179,   181,   183,   185,
   187,   189,   191,   194,   197,   199,   203,   206,   207,   210,
   213,   216,   227,   237,   247,   256,   265,   268,   270,   273,
   277,   279,   281,   283,   285,   287,   289,   291,   293,   297,
   299,   301,   303,   305,   307,   309,   311,   313,   315,   317,
   319,   321,   323,   325,   327,   329,   331,   333,   335,   337,
   339,   342,   344,   346,   348,   351,   353,   354,   356,   358,
   360,   362,   364,   366,   368,   370,   372,   374,   377,   379,
   382,   385
};

static const short yyrhs[] = {    76,
    83,     0,    83,     0,    77,    78,     0,    50,     0,     0,
    16,    52,    79,    53,     0,    80,     0,    79,    80,     0,
    54,   116,    55,   116,    56,     0,     8,    52,    82,    53,
     0,   116,     0,    82,    57,   116,     0,    84,     0,    83,
    84,     0,    95,     0,    90,     0,    91,     0,    81,     0,
    92,     0,    86,     0,    87,     0,    88,     0,   116,     0,
    85,    57,   116,     0,    52,    89,    37,    53,     0,    17,
    47,   116,    55,    85,    58,     0,    17,    47,   116,    58,
     0,    52,    37,    37,    53,     0,    18,     0,    19,     0,
     3,    52,    37,    47,     0,     4,    51,     0,     5,    52,
   116,    59,   121,    60,    93,    53,     0,    94,     0,    93,
    57,   121,     0,   121,     0,    96,    52,   116,   100,    61,
    98,    99,    53,     0,    25,    52,   116,    99,    53,     0,
    97,    43,     0,    97,    22,     0,    97,    20,    21,     0,
    97,    23,     0,    97,    24,     0,    97,    27,     0,     0,
   117,     0,     0,    99,   110,     0,    57,   101,     0,     0,
   112,    39,     0,   112,    40,     0,   112,    42,     0,    59,
   103,    60,     0,     0,   103,    57,   104,     0,   104,     0,
   105,     0,    51,     0,   107,     0,    48,     0,    49,     0,
    62,    48,     0,    62,    49,     0,    48,     0,    54,   106,
    56,     0,    57,   109,     0,     0,   112,    39,     0,   112,
    40,     0,   112,    42,     0,   111,    57,   112,   113,   114,
   108,    61,   117,   118,   119,     0,   111,    57,   112,   113,
   114,   108,    61,   117,   119,     0,   111,    57,   112,   113,
   108,    61,   117,   118,   119,     0,   111,    57,   112,   113,
   108,    61,   117,   119,     0,    26,    57,   112,   113,   108,
    61,   117,   119,     0,   111,    10,     0,   121,     0,   116,
   102,     0,   116,   102,     6,     0,    27,     0,    39,     0,
    40,     0,    41,     0,    42,     0,    44,     0,    46,     0,
    45,     0,    57,    37,    12,     0,    63,     0,    64,     0,
    65,     0,    66,     0,    67,     0,    68,     0,    69,     0,
    61,     0,    70,     0,    62,     0,    71,     0,    72,     0,
    28,     0,    29,     0,    30,     0,    31,     0,    32,     0,
    33,     0,    34,     0,    35,     0,    37,     0,    36,   115,
     0,    38,     0,   116,     0,    47,     0,    45,    38,     0,
    11,     0,     0,    63,     0,    73,     0,    67,     0,    72,
     0,    68,     0,    74,     0,    70,     0,    62,     0,    64,
     0,    71,     0,   121,    37,     0,    37,     0,   121,   120,
     0,    36,   115,     0,   121,    36,   115,     0
};

#endif

#if YYDEBUG != 0
static const short yyrline[] = { 0,
    84,    86,    88,    90,    90,    92,    94,    96,    98,   100,
   102,   104,   106,   108,   111,   113,   115,   117,   119,   121,
   123,   125,   127,   129,   131,   133,   136,   138,   140,   142,
   145,   147,   149,   152,   154,   157,   159,   163,   165,   167,
   169,   171,   173,   175,   177,   182,   185,   187,   190,   192,
   194,   196,   198,   201,   203,   205,   207,   209,   211,   213,
   215,   217,   219,   221,   223,   225,   228,   230,   232,   234,
   236,   238,   241,   244,   247,   250,   253,   255,   257,   259,
   261,   263,   265,   267,   269,   271,   273,   275,   277,   279,
   280,   281,   282,   283,   284,   285,   286,   287,   288,   289,
   290,   291,   292,   293,   294,   295,   296,   297,   298,   300,
   302,   304,   306,   308,   310,   312,   314,   316,   318,   320,
   322,   324,   326,   328,   330,   332,   334,   337,   339,   341,
   343,   345
};

static const char * const yytname[] = {   "$","error","$illegal.","HELP_DEF",
"DEF_HELP_FILE","CONFLICT","WAIT","DEFINED_MENU","DEFINED_ACTION","MENU_ADDITION",
"ITEM_REF","DEFAULT","ITEM_HELP","MENU_LOCAL","MENU_REMOTE","MENU_REMOTE_OPTIONS",
"MAINS","INIT","INCLUDE_LOCAL","INCLUDE_REMOTE","MULTIPLE","USE","DYNAMIC","HISTORY",
"SELECT","ADD_TO_MENU","TEMPLATE","ORPHAN","SHIFT_R","SHIFT_L","LEQ","GEQ","EQU",
"NEQ","AND","OR","OPERATOR","NAME","STRING","LOCAL","REMOTE","LOCAL_REMOTE",
"REMOTE_OPTIONS","MENU","NEW_MENU","HELP_FILE","DYNAMIC_MENU","TEXT","INTEGER_VAL",
"DOUBLE_VAL","USER_COPYRIGHT","EXPLICIT_STRING","'{'","'}'","'['","'='","']'",
"','","';'","'('","')'","':'","'-'","'%'","'/'","'<'","'>'","'^'","'|'","'?'",
"'+'","'*'","'&'","'!'","'$'","Description","Prologue","User","AlternativeMains",
"MainList","MainEntry","DefinedActionList","NameList","ObjectList","Object",
"PriorNameList","IncludeEntry","InitEntry","ClassRelation","Include","HelpDef",
"HelpDefaultFile","ConflictList","ConflictBody","FirstConflict","Menu","MenuId",
"Qualifier","MenuTitle","MenuBody","MenuInit","MenuInitName","Parameters","ParameterList",
"Parameter","Number","PositiveInteger","MenuStackReference","Init","InitName",
"MenuItem","Command","Action","ActionType","HelpRef","Operator","Name","String",
"Help","HelpFile","StringChar","WordString",""
};
#endif

static const short yyr1[] = {     0,
    75,    75,    76,    77,    77,    78,    79,    79,    80,    81,
    82,    82,    83,    83,    84,    84,    84,    84,    84,    84,
    84,    84,    85,    85,    86,    87,    87,    88,    89,    89,
    90,    91,    92,    93,    93,    94,    95,    95,    96,    97,
    97,    97,    97,    97,    97,    98,    99,    99,   100,   100,
   101,   101,   101,   102,   102,   103,   103,   104,   104,   104,
   105,   105,   105,   105,   106,   107,   108,   108,   109,   109,
   109,   110,   110,   110,   110,   110,   110,   111,   112,   112,
   112,   113,   113,   113,   113,   113,   113,   113,   114,   115,
   115,   115,   115,   115,   115,   115,   115,   115,   115,   115,
   115,   115,   115,   115,   115,   115,   115,   115,   115,   116,
   116,   117,   117,   118,   119,   119,   119,   120,   120,   120,
   120,   120,   120,   120,   120,   120,   120,   121,   121,   121,
   121,   121
};

static const short yyr2[] = {     0,
     2,     1,     2,     1,     0,     4,     1,     2,     5,     4,
     1,     3,     1,     2,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     3,     4,     6,     4,     4,     1,     1,
     4,     2,     8,     1,     3,     1,     8,     5,     2,     2,
     3,     2,     2,     2,     0,     1,     0,     2,     2,     0,
     2,     2,     2,     3,     0,     3,     1,     1,     1,     1,
     1,     1,     2,     2,     1,     3,     2,     0,     2,     2,
     2,    10,     9,     9,     8,     8,     2,     1,     2,     3,
     1,     1,     1,     1,     1,     1,     1,     1,     3,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     2,     1,     1,     1,     2,     1,     0,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     2,     1,     2,
     2,     3
};

static const short yydefact[] = {    45,
     0,     0,     0,     0,     0,     0,     4,     0,    45,     0,
    18,    45,    13,    20,    21,    22,    16,    17,    19,    15,
     0,     0,     0,    32,     0,     0,     0,     0,    29,    30,
     0,     0,    45,     0,     3,    14,     0,     0,    40,    42,
    43,    44,    39,     0,     0,   110,     0,     0,    11,     0,
    47,     0,     0,     0,    50,    41,    31,   102,   103,   104,
   105,   106,   107,   108,   109,    97,    99,    90,    91,    92,
    93,    94,    95,    96,    98,   100,   101,   111,     0,    10,
     0,     0,    27,     0,    28,    25,     0,     0,     7,     0,
     0,     0,   129,     0,    12,     0,    23,     0,    38,    48,
     0,    78,     0,     6,     8,    81,    49,     0,    55,     0,
   131,     0,   128,     0,   125,   118,   126,   120,   122,   124,
   127,   121,   119,   123,   130,     0,    26,     0,    77,     0,
     0,    51,    52,    53,     0,    79,   112,    47,   113,    46,
   132,     0,    34,    36,    24,     0,     0,     0,    61,    62,
    59,     0,     0,     0,    57,    58,    60,    80,     0,    33,
     0,    82,    83,    84,    85,    86,    88,    87,    68,    68,
     9,    65,     0,    63,    64,     0,    54,    37,    35,     0,
     0,     0,     0,    68,    66,    56,    67,     0,     0,   110,
     0,     0,    69,    70,    71,   117,    89,   117,     0,   116,
     0,    76,   114,   117,    75,   117,   115,    74,   117,    73,
    72,     0,     0,     0
};

static const short yydefgoto[] = {   212,
     9,    10,    35,    88,    89,    11,    48,    12,    13,    96,
    14,    15,    16,    32,    17,    18,    19,   142,   143,    20,
    21,    22,   138,    84,    91,   107,   136,   154,   155,   156,
   173,   157,   181,   187,   100,   101,   188,   169,   184,    78,
   109,   140,   204,   202,   125,   102
};

static const short yypact[] = {    24,
   -45,   -31,    -4,    17,    46,    44,-32768,    36,    42,   115,
-32768,     6,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
   100,    80,   122,-32768,    72,    72,    72,    72,-32768,-32768,
   128,   147,    13,   138,-32768,-32768,    72,   170,-32768,-32768,
-32768,-32768,-32768,   145,    49,-32768,   134,   -20,-32768,   -33,
-32768,   142,   143,   144,   137,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,    91,-32768,
    72,    72,-32768,    -2,-32768,-32768,    72,    95,-32768,   117,
   136,    49,-32768,    62,-32768,   113,-32768,   146,-32768,-32768,
    -5,   105,   149,-32768,-32768,-32768,-32768,    50,   148,   125,
-32768,    49,-32768,    91,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,    72,-32768,   117,-32768,   117,
    72,-32768,-32768,-32768,    89,   193,-32768,-32768,-32768,-32768,
-32768,    15,-32768,   105,-32768,   141,   141,   150,-32768,-32768,
-32768,   152,   140,    31,-32768,-32768,-32768,-32768,    34,-32768,
    91,-32768,-32768,-32768,-32768,-32768,-32768,-32768,   151,   153,
-32768,-32768,   155,-32768,-32768,    89,-32768,-32768,   105,   117,
   154,   119,   156,   151,-32768,-32768,-32768,   118,   125,   189,
   125,   157,-32768,-32768,-32768,     8,-32768,    -3,   125,-32768,
   164,-32768,-32768,     8,-32768,    -3,-32768,-32768,     8,-32768,
-32768,   205,   209,-32768
};

static const short yypgoto[] = {-32768,
-32768,-32768,-32768,-32768,   124,-32768,-32768,   204,     3,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,    76,-32768,-32768,-32768,-32768,    40,-32768,
-32768,-32768,  -109,-32768,-32768,-32768,   -64,    73,-32768,   -49,
   -25,   -94,    16,   -59,-32768,   -75
};


#define	YYLAST		222


static const short yytable[] = {    47,
    49,    50,    51,    94,   129,    -2,    23,   200,     1,     2,
     3,    55,    -1,     4,    36,     1,     2,     3,   200,    24,
     4,    82,     5,    98,    83,   108,     1,     2,     3,     5,
     6,     4,    80,    92,    93,    36,    81,     6,   144,    -5,
     5,   201,   111,   203,     1,     2,     3,    25,     6,     4,
    99,   130,   201,    29,    30,    95,    97,     8,     5,    98,
   183,   103,   141,   146,     8,   147,     6,   160,    26,    92,
    93,   161,    31,     7,   192,     8,    58,    59,    60,    61,
    62,    63,    64,    65,   139,   179,   178,   176,   132,   133,
   177,   134,    27,     8,   196,    28,   198,   112,   113,    38,
   145,    39,    40,    41,   206,   148,    42,    45,    46,    66,
    67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
    77,   114,    43,   115,   116,   117,    92,    93,   118,   119,
    34,   120,   121,   122,   123,   124,   149,   150,   205,   151,
   112,   113,   152,   106,   208,   106,   210,   104,    87,   211,
   153,    37,    45,    46,    45,   190,   193,   194,    44,   195,
    45,    46,   137,   139,    52,   139,   115,   116,   117,   126,
   127,   118,   119,   139,   120,   121,   122,   123,   124,   162,
   163,   164,   165,    53,   166,   167,   168,   174,   175,    54,
    56,    57,    79,    90,    85,    86,   110,    87,   158,   172,
   197,   207,   128,   131,   213,   171,   135,   180,   214,   182,
   185,   105,    33,   159,   189,   186,   191,   199,     0,   170,
     0,   209
};

static const short yycheck[] = {    25,
    26,    27,    28,    79,    10,     0,    52,    11,     3,     4,
     5,    37,     0,     8,    12,     3,     4,     5,    11,    51,
     8,    55,    17,    26,    58,    90,     3,     4,     5,    17,
    25,     8,    53,    36,    37,    33,    57,    25,   114,    16,
    17,    45,    92,    47,     3,     4,     5,    52,    25,     8,
    53,    57,    45,    18,    19,    81,    82,    52,    17,    26,
   170,    87,   112,   128,    52,   130,    25,    53,    52,    36,
    37,    57,    37,    50,   184,    52,    28,    29,    30,    31,
    32,    33,    34,    35,   110,   161,    53,    57,    39,    40,
    60,    42,    47,    52,   189,    52,   191,    36,    37,    20,
   126,    22,    23,    24,   199,   131,    27,    36,    37,    61,
    62,    63,    64,    65,    66,    67,    68,    69,    70,    71,
    72,    60,    43,    62,    63,    64,    36,    37,    67,    68,
    16,    70,    71,    72,    73,    74,    48,    49,   198,    51,
    36,    37,    54,    27,   204,    27,   206,    53,    54,   209,
    62,    52,    36,    37,    36,    37,    39,    40,    37,    42,
    36,    37,    38,   189,    37,   191,    62,    63,    64,    57,
    58,    67,    68,   199,    70,    71,    72,    73,    74,    39,
    40,    41,    42,    37,    44,    45,    46,    48,    49,    52,
    21,    47,    59,    57,    53,    53,    61,    54,     6,    48,
    12,    38,    57,    55,     0,    56,    59,    57,     0,    57,
    56,    88,     9,   138,    61,   176,    61,    61,    -1,   147,
    -1,   206
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/lib/bison.simple"

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Bob Corbett and Richard Stallman

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#ifndef alloca
#ifdef __GNUC__
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi)
#include <alloca.h>
#else /* not sparc */
#if defined (MSDOS) && !defined (__TURBOC__)
#include <malloc.h>
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
#include <malloc.h>
 #pragma alloca
#else /* not MSDOS, __TURBOC__, or _AIX */
#ifdef __hpux
#ifdef __cplusplus
extern "C" {
void *alloca (unsigned int);
};
#else /* not __cplusplus */
void *alloca ();
#endif /* not __cplusplus */
#endif /* __hpux */
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc.  */
#endif /* not GNU C.  */
#endif /* alloca not defined.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	return(0)
#define YYABORT 	return(1)
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#define YYLEX		yylex(&yylval, &yylloc)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
int yyparse (void);
#endif

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_bcopy(FROM,TO,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_bcopy (from, to, count)
     char *from;
     char *to;
     int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_bcopy (char *from, char *to, int count)
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

#line 184 "/usr/lib/bison.simple"
int
yyparse()
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if yyoverflow is a macro.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yystacksize);
#else
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yystacksize);
#endif

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
      yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
      __yy_bcopy ((char *)yyss1, (char *)yyss, size * sizeof (*yyssp));
      yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
      __yy_bcopy ((char *)yyvs1, (char *)yyvs, size * sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
      __yy_bcopy ((char *)yyls1, (char *)yyls, size * sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

  goto yybackup;
 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  if (yylen > 0)
    yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 4:
#line 90 "../menu_b.y"
{SetUserCopyright();;
    break;}
case 5:
#line 90 "../menu_b.y"
{NoUserCopyright();;
    break;}
case 9:
#line 98 "../menu_b.y"
{MainEntry(yyvsp[-3].Val,yyvsp[-1].Val);;
    break;}
case 11:
#line 102 "../menu_b.y"
{DefinedName(yyvsp[0].Val);;
    break;}
case 12:
#line 104 "../menu_b.y"
{DefinedName(yyvsp[0].Val);;
    break;}
case 23:
#line 127 "../menu_b.y"
{yyval.Val=MakeNameList(yyvsp[0].Val);;
    break;}
case 24:
#line 129 "../menu_b.y"
{yyval.Val=NameListAppend(yyvsp[-2].Val,yyvsp[0].Val);;
    break;}
case 25:
#line 131 "../menu_b.y"
{FoundInclude(yyvsp[-2].Int,yyvsp[-1].Name);;
    break;}
case 26:
#line 134 "../menu_b.y"
{FoundInitCode(yyvsp[-4].Text,yyvsp[-3].Val,yyvsp[-1].Val);;
    break;}
case 27:
#line 136 "../menu_b.y"
{FoundInitCode(yyvsp[-2].Text,yyvsp[-1].Val,0);;
    break;}
case 28:
#line 138 "../menu_b.y"
{FoundClassRelation(yyvsp[-2].Name,yyvsp[-1].Name);;
    break;}
case 29:
#line 140 "../menu_b.y"
{yyval.Int=INCLUDE_LOCAL;;
    break;}
case 30:
#line 142 "../menu_b.y"
{yyval.Int=INCLUDE_REMOTE;;
    break;}
case 31:
#line 145 "../menu_b.y"
{FoundHelpDef(yyvsp[-1].Name,yyvsp[0].Text);;
    break;}
case 32:
#line 147 "../menu_b.y"
{FoundHelpDefaultFile(yyvsp[0].Name);;
    break;}
case 33:
#line 150 "../menu_b.y"
{Conflict(yyvsp[-5].Val,yyvsp[-3].Val,yyvsp[-1].Val);;
    break;}
case 34:
#line 152 "../menu_b.y"
{ yyval.Val = FirstConflict(yyvsp[0].Val);;
    break;}
case 35:
#line 154 "../menu_b.y"
{ yyvsp[-2].Val->Strings->Append(yyvsp[0].Val->Name);
						delete yyvsp[0].Val; ;
    break;}
case 37:
#line 159 "../menu_b.y"
{

						Menu(*yyvsp[-7].TheMenuType,yyvsp[-5].Val,yyvsp[-4].Val,yyvsp[-2].Val,yyvsp[-1].Val);;
    break;}
case 38:
#line 163 "../menu_b.y"
{MenuAddition(yyvsp[-2].Val,yyvsp[-1].Val);;
    break;}
case 40:
#line 167 "../menu_b.y"
{yyvsp[-1].TheMenuType->SetDynamic(); yyval.TheMenuType=yyvsp[-1].TheMenuType;;
    break;}
case 41:
#line 169 "../menu_b.y"
{yyvsp[-2].TheMenuType->SetMultiUse(); yyval.TheMenuType=yyvsp[-2].TheMenuType;;
    break;}
case 42:
#line 171 "../menu_b.y"
{yyvsp[-1].TheMenuType->SetHistory(); yyval.TheMenuType=yyvsp[-1].TheMenuType;;
    break;}
case 43:
#line 173 "../menu_b.y"
{yyvsp[-1].TheMenuType->SetSelect(); yyval.TheMenuType=yyvsp[-1].TheMenuType;;
    break;}
case 44:
#line 175 "../menu_b.y"
{yyvsp[-1].TheMenuType->SetOrphan(); yyval.TheMenuType=yyvsp[-1].TheMenuType;;
    break;}
case 45:
#line 177 "../menu_b.y"
{MenuType * New = new MenuType;
						/* above (=0) garbage avoids 
						 * bad warning message */
						New->Clear(); yyval.TheMenuType=New;;
    break;}
case 47:
#line 185 "../menu_b.y"
{yyval.Val=StartItemList();;
    break;}
case 48:
#line 187 "../menu_b.y"
{yyvsp[-1].Val->List->Append(yyvsp[0].Val->Item) ;
						yyval.Val=yyvsp[-1].Val; delete yyvsp[0].Val;;
    break;}
case 49:
#line 190 "../menu_b.y"
{yyval.Val=yyvsp[0].Val;;
    break;}
case 50:
#line 192 "../menu_b.y"
{yyval.Val=MakeNullValue();;
    break;}
case 51:
#line 194 "../menu_b.y"
{yyval.Val=MakeMenuInit(yyvsp[-1].Tree,MENU_LOCAL);;
    break;}
case 52:
#line 196 "../menu_b.y"
{yyval.Val=MakeMenuInit(yyvsp[-1].Tree,MENU_REMOTE);;
    break;}
case 53:
#line 198 "../menu_b.y"
{yyval.Val=MakeMenuInit(
						yyvsp[-1].Tree,MENU_REMOTE_OPTIONS);;
    break;}
case 54:
#line 201 "../menu_b.y"
{yyval.Tree=yyvsp[-1].Tree;;
    break;}
case 55:
#line 203 "../menu_b.y"
{yyval.Tree=MakeParameterList();;
    break;}
case 56:
#line 205 "../menu_b.y"
{yyval.Tree=MakeParameterList(yyvsp[-2].Tree,yyvsp[0].Tree);;
    break;}
case 57:
#line 207 "../menu_b.y"
{yyval.Tree=MakeParameterList(yyvsp[0].Tree);;
    break;}
case 58:
#line 209 "../menu_b.y"
{yyval.Tree=yyvsp[0].Tree;;
    break;}
case 59:
#line 211 "../menu_b.y"
{yyval.Tree=MakeParameter(yyvsp[0].Name);;
    break;}
case 60:
#line 213 "../menu_b.y"
{yyval.Tree=MakeStackParameter(yyvsp[0].Int);;
    break;}
case 61:
#line 215 "../menu_b.y"
{yyval.Tree=MakeNumber(yyvsp[0].Int);;
    break;}
case 62:
#line 217 "../menu_b.y"
{yyval.Tree=MakeNumber(yyvsp[0].Dbl);;
    break;}
case 63:
#line 219 "../menu_b.y"
{yyval.Tree=MakeNegNumber(yyvsp[0].Int);;
    break;}
case 64:
#line 221 "../menu_b.y"
{yyval.Tree=MakeNegNumber(yyvsp[0].Dbl);;
    break;}
case 66:
#line 225 "../menu_b.y"
{yyval.Int=yyvsp[-1].Int;;
    break;}
case 67:
#line 228 "../menu_b.y"
{yyval.Val=yyvsp[0].Val;;
    break;}
case 68:
#line 230 "../menu_b.y"
{yyval.Val=MakeNullValue();;
    break;}
case 69:
#line 232 "../menu_b.y"
{yyval.Val=MakeMenuInit(yyvsp[-1].Tree,yyvsp[0].Int);;
    break;}
case 70:
#line 234 "../menu_b.y"
{yyval.Val=MakeMenuInit(yyvsp[-1].Tree,yyvsp[0].Int);;
    break;}
case 71:
#line 236 "../menu_b.y"
{yyval.Val=MakeMenuInit(yyvsp[-1].Tree,yyvsp[0].Int);;
    break;}
case 72:
#line 238 "../menu_b.y"
{
				yyval.Val=FoundMenuItem(yyvsp[-9].Val,yyvsp[-7].Tree,yyvsp[-6].Val,yyvsp[-5].Name,yyvsp[-4].Val,yyvsp[-2].Val,yyvsp[-1].Text,yyvsp[0].Name); ;
    break;}
case 73:
#line 241 "../menu_b.y"
{
				yyval.Val=FoundMenuItem(yyvsp[-8].Val,yyvsp[-6].Tree,yyvsp[-5].Val,yyvsp[-4].Name,yyvsp[-3].Val,yyvsp[-1].Val,0,yyvsp[0].Name);;
    break;}
case 74:
#line 244 "../menu_b.y"
{
				yyval.Val=FoundMenuItem(yyvsp[-8].Val,yyvsp[-6].Tree,yyvsp[-5].Val,0,yyvsp[-4].Val,yyvsp[-2].Val,yyvsp[-1].Text,yyvsp[0].Name); ;
    break;}
case 75:
#line 247 "../menu_b.y"
{
				yyval.Val=FoundMenuItem(yyvsp[-7].Val,yyvsp[-5].Tree,yyvsp[-4].Val,0,yyvsp[-3].Val,yyvsp[-1].Val,0,yyvsp[0].Name);;
    break;}
case 76:
#line 250 "../menu_b.y"
{
				yyval.Val=FoundItemTemplate(yyvsp[-5].Tree,yyvsp[-4].Val,0,yyvsp[-3].Val,yyvsp[-1].Val,0,yyvsp[0].Name);;
    break;}
case 77:
#line 253 "../menu_b.y"
{yyval.Val=FoundItemRef(yyvsp[-1].Val);;
    break;}
case 78:
#line 255 "../menu_b.y"
{Found("Command");;
    break;}
case 79:
#line 257 "../menu_b.y"
{yyval.Tree=MakeActionValue(yyvsp[-1].Val,yyvsp[0].Tree);;
    break;}
case 80:
#line 259 "../menu_b.y"
{yyval.Tree=MakeActionValue(yyvsp[-2].Val,yyvsp[-1].Tree,1);;
    break;}
case 81:
#line 261 "../menu_b.y"
{yyval.Tree=MakeActionValue("Orphan");;
    break;}
case 82:
#line 263 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Int);;
    break;}
case 83:
#line 265 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Int);;
    break;}
case 84:
#line 267 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Int);;
    break;}
case 85:
#line 269 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Int);;
    break;}
case 86:
#line 271 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Int);;
    break;}
case 87:
#line 273 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Int);;
    break;}
case 88:
#line 275 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Int);;
    break;}
case 89:
#line 277 "../menu_b.y"
{yyval.Name=yyvsp[-1].Name;;
    break;}
case 90:
#line 279 "../menu_b.y"
{yyval.Name=MakeOperatorString('%');;
    break;}
case 91:
#line 280 "../menu_b.y"
{yyval.Name=MakeOperatorString('/');;
    break;}
case 92:
#line 281 "../menu_b.y"
{yyval.Name=MakeOperatorString('<');;
    break;}
case 93:
#line 282 "../menu_b.y"
{yyval.Name=MakeOperatorString('>');;
    break;}
case 94:
#line 283 "../menu_b.y"
{yyval.Name=MakeOperatorString('^');;
    break;}
case 95:
#line 284 "../menu_b.y"
{yyval.Name=MakeOperatorString('|');;
    break;}
case 96:
#line 285 "../menu_b.y"
{yyval.Name=MakeOperatorString('?');;
    break;}
case 97:
#line 286 "../menu_b.y"
{yyval.Name=MakeOperatorString(':');;
    break;}
case 98:
#line 287 "../menu_b.y"
{yyval.Name=MakeOperatorString('+');;
    break;}
case 99:
#line 288 "../menu_b.y"
{yyval.Name=MakeOperatorString('-');;
    break;}
case 100:
#line 289 "../menu_b.y"
{yyval.Name=MakeOperatorString('*');;
    break;}
case 101:
#line 290 "../menu_b.y"
{yyval.Name=MakeOperatorString('&');;
    break;}
case 102:
#line 291 "../menu_b.y"
{yyval.Name=MakeOperatorString(SHIFT_R);;
    break;}
case 103:
#line 292 "../menu_b.y"
{yyval.Name=MakeOperatorString(SHIFT_L);;
    break;}
case 104:
#line 293 "../menu_b.y"
{yyval.Name=MakeOperatorString(LEQ);;
    break;}
case 105:
#line 294 "../menu_b.y"
{yyval.Name=MakeOperatorString(GEQ);;
    break;}
case 106:
#line 295 "../menu_b.y"
{yyval.Name=MakeOperatorString(EQU);;
    break;}
case 107:
#line 296 "../menu_b.y"
{yyval.Name=MakeOperatorString(NEQ);;
    break;}
case 108:
#line 297 "../menu_b.y"
{yyval.Name=MakeOperatorString(AND);;
    break;}
case 109:
#line 298 "../menu_b.y"
{yyval.Name=MakeOperatorString(OR);;
    break;}
case 110:
#line 300 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Name);
							Found("Name");;
    break;}
case 111:
#line 302 "../menu_b.y"
{yyval.Val=MakeValue(MakeOperatorName(yyvsp[0].Name));;
    break;}
case 112:
#line 304 "../menu_b.y"
{yyval.Val=MakeValue(yyvsp[0].Name);
							Found("String");;
    break;}
case 113:
#line 306 "../menu_b.y"
{Found("WordStringString");;
    break;}
case 115:
#line 310 "../menu_b.y"
{yyval.Name=yyvsp[0].Name;;
    break;}
case 116:
#line 312 "../menu_b.y"
{yyval.Name=MakeName(the_help_default_file);;
    break;}
case 117:
#line 314 "../menu_b.y"
{yyval.Name=0;;
    break;}
case 118:
#line 316 "../menu_b.y"
{yyval.Int='%';;
    break;}
case 119:
#line 318 "../menu_b.y"
{yyval.Int='!';;
    break;}
case 120:
#line 320 "../menu_b.y"
{yyval.Int='^';;
    break;}
case 121:
#line 322 "../menu_b.y"
{yyval.Int='&';;
    break;}
case 122:
#line 324 "../menu_b.y"
{yyval.Int='|';;
    break;}
case 123:
#line 326 "../menu_b.y"
{yyval.Int='$';;
    break;}
case 124:
#line 328 "../menu_b.y"
{yyval.Int='+';;
    break;}
case 125:
#line 330 "../menu_b.y"
{yyval.Int='-';;
    break;}
case 126:
#line 332 "../menu_b.y"
{yyval.Int='/';;
    break;}
case 127:
#line 334 "../menu_b.y"
{yyval.Int='*';;
    break;}
case 128:
#line 337 "../menu_b.y"
{yyval.Val=AddToWordString(yyvsp[-1].Val,yyvsp[0].Name);;
    break;}
case 129:
#line 339 "../menu_b.y"
{yyval.Val=CreateWordString(yyvsp[0].Name);;
    break;}
case 130:
#line 341 "../menu_b.y"
{yyval.Val=ConcatWordString(yyvsp[-1].Val,yyvsp[0].Int);;
    break;}
case 131:
#line 343 "../menu_b.y"
{yyval.Val=MakeValue(MakeOperatorName(yyvsp[0].Name));;
    break;}
case 132:
#line 346 "../menu_b.y"
{yyval.Val=MakeValue(MakeOperatorCommand(yyvsp[-2].Val->Name,yyvsp[0].Name));;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 465 "/usr/lib/bison.simple"

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -yyn if nec to avoid negative indexes in yycheck.  */
	  for (x = (yyn < 0 ? -yyn : 0);
	       x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (yyn < 0 ? -yyn : 0);
		       x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

  goto yyerrlab1;
yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;
}
#line 348 "../menu_b.y"
