/*
 *  dyntext.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef DYNTEXT_DOT_H
#define DYNTEXT_DOT_H
/*  dyntext.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
class DynamicTextControl {
public:
	DynamicTextControl() ;
	virtual void TextServer(struct PacketHeader& , const char *);
};


enum TextControlCommands {TextControlGetWindow,TextControlOpenWindow,
	TextControlSendHandle,TextControlSendOpen,TextControlNewSampleRate,
	TextControlRaiseWindow};

#endif /* #ifdef DYNTEXT_DOT_H */
