/*
 *  nodemenu.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  nodemenu.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
static const char * MenuDirectory =
	"/usra/paul/develop/generic/menus/nodes/" ;

// used for writing menu input files that are later read by menu processor

// static const char * RemoteMenuDirectory =
//	"/usra/paul/develop/generic/menus/dspgui/" ;

// used for writing remote interface to remote routines invoked through menus

// static const char * LocalMenuDirectory =
//	"/usra/paul/develop/generic/menus/cgi/" ;

// used for writing local interface to remote routines invoked through menus

