/*
 *  toint.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */

#ifndef TOINT_DOT_H
#define TOINT_DOT_H

#include "ObjProDSP/portable.h"
#include "ObjProDSP/entenm.h"
#include "ObjProGen/debug.h"
#include "ObjProUsr/procstr.h"

class ToInteger: public ProcessNodeStr {
#line 24 "../toint.usr"
 
    int32 over_min ;
    int32 over_max ;
#line 24 "/dist/opd_2/opd-0.2/src/include/ObjProDSPint/ObjProUsr/toint.h"
public:
	ToInteger (const char * Name);
	virtual ~ToInteger();
	virtual int CheckSafeDelete() ;
	virtual ErrCode DoNode(int32);
	void Describe(OutTokens& Out, ListEntity Option);
	virtual int CppList(OutTokens& Out, CppListCmds Cmd);
};


#endif /* #ifdef TOINT_DOT_H */
