#!/usr/bin/perl
#  padcat.pl from ObjectProDSP 0.2
#  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
#  Licensed for free use and distribution under version 2 of the Gnu General
#  Public License. Please see file COPYING for details and restrictions.
#  
#  ObjectProDSP is a trademark of Mountain Math Software.
#  
open(in_file,$ARGV[0]) || die("Cannot open `$ARGV[0]': $!.\n");
# open(out_file,">".$ARGV[1]) || die("Cannot create `$ARGV[1]: $!'.\n");
print "\n        Following is the DSP++ code for this example:\01\n\n" ;
while($line= <in_file>) {
	chop($line) ;
	print "$line\01\n" ;
	#print out_file "$line  \n" ;
}
