/*  sigmem.C   */
/*  Copyright 1990 Mountain Math Software  */
/*  All Rights Reserved                    */
#include "signode.h"

void SignalStr::Raise()
{
    raise_window();
}

void SignalStr::Edit()
{
	add_to_net_graph();
}

void SignalStr::SetSampleRate(double Rate, int16 Channel)
{
	DfNode::SetSampleRate(Rate,Channel);
}
void SignalStr::DisplayOutputTiming(int16 Channel)
{
	DfNode::DisplayOutputTiming(Channel);
}

void SignalStr::NextFreeOutput()
{
	DfNode::NextFreeOutput();
}

void SignalStr::Unlink()
{
    DfNode::Unlink();
}



