#include "ObjProNet/dfnode.h"

#include "ObjProUsr/display.h"
#include "ObjProGen/outtok.h"
#include "ObjProGui/intfc.h"
#include "ObjProGui/remmen.h"
#include "ObjProGui/user.h"
#include "ObjProGui/dynmnu.h"
#include "ObjProArith/typout.h"
#include "ObjProGui/array.h"
#include "ObjProGui/interinit.h"
#include "ObjProGen/stattyp.h"
#include "ObjProGui/strmstr.h"
static EntityList * DisplayNodeNodeList = 0 ;
static InteractiveEntity * IntEntDisplayNode ;
void DisplayNodeNodesInit();
DisplayNode::DisplayNode (const char * Name, int16 proc_in, int32 procDeltaIn, 
		int32 procEltSz, int32 procBlockSize, 
		TimingType procTheTimingType, 
		ArithType::ArithCapabilities procArithTypeIn):
	DisplayNodeStr(Name, proc_in, procDeltaIn, 
	 new StreamStr(procEltSz , procBlockSize , procArithTypeIn), procTheTimingType)
,
	proc_in_1(proc_in),
	procDeltaIn_2(procDeltaIn),
	procEltSz_3(procEltSz),
	procBlockSize_4(procBlockSize),
	procTheTimingType_5(procTheTimingType),
	procArithTypeIn_6(procArithTypeIn)
{
} // end constructor

DisplayNode::~DisplayNode()
{
} // end destructor

int DisplayNode::CheckSafeDelete()
{
	int Safe_Check_Return = DfNode::CheckSafeDelete();
	if (!Safe_Check_Return) return 0;
	return 1;
} // end check safe delete

void DisplayNodeNodesInit()
{
	if (DisplayNodeNodeList)  return ;
	DisplayNodeNodeList = new EntityList;
	IntEntDisplayNode = new InteractiveEntity("DisplayNode", DisplayNodeNodeList,
		0, InteractiveNode, "display.h",
		0, "DisplayNodeStr");
	TheNodes->Append(IntEntDisplayNode);
} // end initalization

static InitObj LocalInit(DisplayNodeNodesInit, "DisplayNode", "DisplayNodeStr");

