/*
 *  lcrmexe.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef LCRMEXE_DOT_H 
#define LCRMEXE_DOT_H
#include "menucmd.h"
CommandParameters DoSaveExit_Rem_Call_Struct = {
	-1, 0, 0, 0, 0, 0
};
CommandParameters DoExit_Rem_Call_Struct = {
	-2, 0, 0, 0, 0, 0
};
#endif /* #ifndef MENEXE_DOT_H*/
