#include "ObjProNet/dfnode.h"

#include "ObjProUsr/node.h"
#include "ObjProGen/outtok.h"
#include "ObjProGui/intfc.h"
#include "ObjProGui/remmen.h"
#include "ObjProGui/user.h"
#include "ObjProGui/dynmnu.h"
#include "ObjProArith/typout.h"
#include "ObjProGui/array.h"
#include "ObjProGui/interinit.h"
#include "ObjProGen/stattyp.h"
static EntityList * NodeNodeList = 0 ;
static InteractiveEntity * IntEntNode ;
void NodeNodesInit();
Node::Node (const char * Name, int16 in, int16 out, StreamStr*str_in, 
		StreamStr*str_out, int32 NodeDelay, int32 DeltaIn, 
		int32 DeltaOut, int32 Overlap, int32 DelayIn, 
		TimingType TheTimingType):
	DfNode(Name, in, out, str_in, str_out, NodeDelay, DeltaIn, DeltaOut, 
	Overlap, DelayIn, TheTimingType)
,
	in_1(in),
	out_2(out),
	str_in_3(str_in),
	str_out_4(str_out),
	NodeDelay_5(NodeDelay),
	DeltaIn_6(DeltaIn),
	DeltaOut_7(DeltaOut),
	Overlap_8(Overlap),
	DelayIn_9(DelayIn),
	TheTimingType_10(TheTimingType)
{
} // end constructor

Node::~Node()
{
} // end destructor

int Node::CheckSafeDelete()
{
	int Safe_Check_Return = DfNode::CheckSafeDelete();
	if (!Safe_Check_Return) return 0;
	return 1;
} // end check safe delete

void NodeNodesInit()
{
	if (NodeNodeList)  return ;
	NodeNodeList = new EntityList;
	IntEntNode = new InteractiveEntity("Node", NodeNodeList,
		0, InteractiveNode, "node.h",
		0, "DfNode");
	TheNodes->Append(IntEntNode);
} // end initalization

static InitObj LocalInit(NodeNodesInit, "Node", "DfNode");

