/*
 *  menstat.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#include "userwin.h"
// Following is state variables set in the menus
// That influence subsequent menu routines
struct MenuState {
	UserWindowOptions WindowToChangeColor ;
	enum Actions {None,Background,Text,ShowSaturation,ChangeSaturation}
		TheAction ;
	int PlotFlag ;
	int Color ;
	MenuState() ;
	void Clear();
} ;

