#!/bin/sh
#  spell.sh from ObjectProDSP 0.2
#  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
#  Licensed for free use and distribution under version 2 of the Gnu General
#  Public License. Please see file COPYING for details and restrictions.
#  
#  ObjectProDSP is a trademark of Mountain Math Software.
#  
if [ -e spellerr ] ;then rm spellerr ;fi
for i in $* ;do \
        echo "Possible spelling errors in \`$i'" >> spellerr; \
        ispell -l -t < $i | sort | uniq  >> spellerr ;done 

