/*
 *  vocstr.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */

#ifndef VOCSTR_DOT_H
#define VOCSTR_DOT_H
#define __DSP_PP_TARGET_CODE__
#include "ObjProArith/hrdarth.h"

#include "ObjProDSP/portable.h"
#include "ObjProDSP/entenm.h"
#include "ObjProGen/debug.h"
#include "ObjProDSPcom/tarnod.h"
class VoiceStripOut: public TargetNode {
#line 61 "../vocstr.usr"
 
	int TheFile ;
	char * to_delete ;
	class VoiceStripOutputFile * TheOutputFile ;
	ErrCode node_state ;
#ifdef INTERACTIVE
	int InitAfterLinked(){return set_read_binary_flag(binary_read_flag) ;}
#endif
#line 30 "/dist/opd_2/opd-0.2/src/include/ObjProDSPtar/ObjProUsr/vocstr.h"
	int binary_read_flag ;
	const char *  FileName_1;
public:
	VoiceStripOut (const char * Name, const char * FileName,
		DfNodeInLink* in_links, DfNodeOutLink* out_links,
		int ** exec_seq,  NodeExecuteType exec_type,
		int delay,
		int VoiceStripOutStaticInitbinary_read_flag);
	virtual ~VoiceStripOut();
	const char * GetFileName() const {return FileName_1;}
	virtual ErrCode DoNode(int32);
	virtual double TimeFirst(DfNodeInLink *,DfNodeOutLink *);
};

extern VoiceStripOut * VoiceStripOutDef;


#endif /* #ifdef VOCSTR_DOT_H */
