/*  hrdinp.c   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef TI_C30
#include <stream.h>
#include "hrdinp.h"

istream& operator>>(istream& o, MachWord& v)
{
	double x ;
	o >> x ;
	v = x ;
	return o ;
}

istream& operator>>(istream& o, AccMachWord& v)
{
	double x ;
	o >> x ;
	v = x ;
	return o ;
}

#ifndef __NT_VC__
istream& operator>>(istream& o, CxMachWord& v)
{
	complex x ;
	o >> x ;
	v = CxMachWord(real(x),imag(x)) ;
	return o ;
}

istream& operator>>(istream& o, CxAccMachWord& v)
{
	complex x ;
	o >> x ;
	v = CxAccMachWord((AccMachWord)real(x),(AccMachWord)imag(x)) ;
	return o ;
}
#endif

#endif
