/*
 *  mknod_b.tab.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
typedef union {
	const char * Name ;
	int32 Int ;
	const char * String ;
	class TextFragmentList * Text ;
	struct Double * Dbl ;
	Complex * Cmplx ;
	int Terminal ;
	struct Compound * Tree ;
	} YYSTYPE;
#define	MENU_LINE	258
#define	BASE_DESCRIPTION	259
#define	DESCRIPTION	260
#define	PARAMETER_VALUE	261
#define	EXPRESSION	262
#define	WAIT	263
#define	INSTANCE	264
#define	TIMING	265
#define	CONSTRUCTOR	266
#define	KERNEL	267
#define	DESTRUCTOR	268
#define	HELP_FILE	269
#define	STATIC_INIT	270
#define	STATIC_DECLARE	271
#define	ARITH_TYPE_PREAMBLE	272
#define	NO_DEFAULT	273
#define	ARROW	274
#define	DOUBLE_COLON	275
#define	INTERACTIVE_LIST	276
#define	STATE_EMIT	277
#define	SAFE_DELETE	278
#define	HELP_DEF_FILE	279
#define	USER_COPYRIGHT	280
#define	CLASS	281
#define	PUBLIC	282
#define	OPERATOR	283
#define	BASE	284
#define	CHECK_PARAMETER	285
#define	PRIVATE	286
#define	PROTECTED	287
#define	C_INCLUDE	288
#define	H_INCLUDE	289
#define	c_INCLUDE	290
#define	h_INCLUDE	291
#define	IC_INCLUDE	292
#define	IH_INCLUDE	293
#define	Ic_INCLUDE	294
#define	Ih_INCLUDE	295
#define	TC_INCLUDE	296
#define	TH_INCLUDE	297
#define	Tc_INCLUDE	298
#define	Th_INCLUDE	299
#define	FIRST_DEFAULT	300
#define	NO_DEFAULT_INSTANCE	301
#define	EMBEDDED_TEXT_LINE	302
#define	EMBEDDED_TEXT	303
#define	CPP_CODE	304
#define	INT	305
#define	INT16	306
#define	INT32	307
#define	MAX	308
#define	MIN	309
#define	MAX_NEG	310
#define	MIN_POS	311
#define	DOUBLE	312
#define	COMPLEX	313
#define	CHANGEABLE	314
#define	MACH_WORD	315
#define	CX_MACH_WORD	316
#define	ACC_MACH_WORD	317
#define	CX_ACC_MACH_WORD	318
#define	SC_MACH_WORD	319
#define	SC_CX_MACH_WORD	320
#define	SC_ACC_MACH_WORD	321
#define	SC_CX_ACC_MACH_WORD	322
#define	STRING	323
#define	NETWORK_REF	324
#define	NETWORK_STATE	325
#define	VOID	326
#define	NAME	327
#define	WORD_STRING	328
#define	INTEGER_VAL	329
#define	NEW	330
#define	DOUBLE_VAL	331
#define	STRING_VAL	332
#define	ANGLE_VAL	333
#define	CHARACTER_VAL	334
#define	TARGET_TYPE	335
#define	OR	336
#define	AND	337
#define	EQU	338
#define	NEQ	339
#define	LEQ	340
#define	GEQ	341
#define	SHIFT_R	342
#define	SHIFT_L	343


extern YYSTYPE yylval;
