#include "menstat.h"
#include "cgidbg.h"

void MenuState::Clear()
{
	WindowToChangeColor = UserWindowNotInitialized;
	TheAction=None;
	Color = 0 ;
	PlotFlag = 0 ;
	// LogOut << "MenuState::Clear exit\n" ;
}

MenuState::MenuState()
{
	// LogOut << "MenuState::ctor\n" ;
	Clear();
}

