/*
 *  ndconst.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  ndconst.h   */
/*  Copyright 1991 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef NDCONST_DOT_H
#define NDCONST_DOT_H
const LARGE_16 = 32767 ;
const LARGE_32 = 0x7fffffff ;
#define LARGE_DOUBLE 1.e100
#define LARGE_DOUBLE_STRING "1.e100"
#define LARGE_NEGATIVE_DOUBLE_STRING "-1.e100"

#endif /* #ifdef NDCONST_DOT_H */
