#include "infile.h"
#include "innode.h"
#include "drvintfc.h"
#include "cgidbg.h" 


int32 DefaultDriverInputInterface(InputNode * TheNode,int Channel,
	int32 Size, MachWord * Data)
{
	// LogOut<<"DefaultDriverInputInterface entry\n" ;
	if (Channel == ResetReaderSystemInterface) return  0 ; 
	InputFile * In = TheNode->GetInputFile() ;
	return In->Read(Channel,Size,Data) ;
}

