extern "C" void perror(const char * dummy);

void this_is_a_dummy_to_force_linkage()
{
	perror("dummy");
}
/*****************

#include <complex.h>
#include <ostream.h>

ostream& operator<< (ostream& out, __complex<double> cx)
{
	out << "(" << real(cx) << ", " << imag(cx) << ")" ;
	return out ;
}

istream& operator>> (istream& in, __complex<double>& cx)
{
	return in ;
}

complex operator/(__complex<double> dv, double d)
{
	return complex(real(dv)/d, imag(dv)/d) ;
}


**********/
