/*  vocfilc.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */

#include "ObjProArith/hrdarth.h"
#include <stream.h>
#include <fcntl.h>
#include "ObjProComGui/cgidbg.h"
#include "ObjProDSP/sysconst.h"
#include "ObjProDSP/sysintfc.h"
#include "ObjProGui/yacintfc.h"
#include "ObjProDSPcom/vocfile.h"


#ifdef INTERACTIVE
void VoiceNode::DisplayHeader()
{
	if (TheInputFile) TheInputFile->DisplayHeader();
	else *Output + OutputHelp <<
		"The current file, `" << GetFileName() <<
		"' could not be read.\n" ;
}
#endif

