#include "ObjProNet/dfnode.h"

#include "ObjProUsr/blkpltstr.h"
#include "ObjProGen/outtok.h"
#include "ObjProGui/intfc.h"
#include "ObjProGui/remmen.h"
#include "ObjProGui/user.h"
#include "ObjProGui/dynmnu.h"
#include "ObjProArith/typout.h"
#include "ObjProGui/array.h"
#include "ObjProGui/interinit.h"
#include "ObjProGen/stattyp.h"
static EntityList * GenericBlockPlotStrNodeList = 0 ;
static InteractiveEntity * IntEntGenericBlockPlotStr ;
void GenericBlockPlotStrNodesInit();
GenericBlockPlotStr::GenericBlockPlotStr (const char * Name, StreamStr*StreamIn, 
		const char * Caption, double MinY, double MaxY, double MinX, 
		double MaxX, int16 NumberBlocks, int16 ScaleFlag, 
		TimingType TheTimingType, PlotDynType DynamicType):
	GenericPlotStr(Name, 1, StreamIn, Caption, MinY, MaxY, MinX, MaxX, 
	NumberBlocks, ScaleFlag, TheTimingType, DynamicType, 0.000000e+00, PlotYs)
,
	StreamIn_1(StreamIn),
	Caption_2(Caption),
	MinY_3(MinY),
	MaxY_4(MaxY),
	MinX_5(MinX),
	MaxX_6(MaxX),
	NumberBlocks_7(NumberBlocks),
	ScaleFlag_8(ScaleFlag),
	TheTimingType_9(TheTimingType),
	DynamicType_10(DynamicType)
{
} // end constructor

GenericBlockPlotStr::~GenericBlockPlotStr()
{
} // end destructor

int GenericBlockPlotStr::CheckSafeDelete()
{
	int Safe_Check_Return = DfNode::CheckSafeDelete();
	if (!Safe_Check_Return) return 0;
	return 1;
} // end check safe delete

void GenericBlockPlotStrNodesInit()
{
	if (GenericBlockPlotStrNodeList)  return ;
	GenericBlockPlotStrNodeList = new EntityList;
	IntEntGenericBlockPlotStr = new InteractiveEntity("GenericBlockPlotStr", GenericBlockPlotStrNodeList,
		0, InteractiveNode, "blkpltstr.h",
		0, "GenericPlotStr");
	TheNodes->Append(IntEntGenericBlockPlotStr);
} // end initalization

static InitObj LocalInit(GenericBlockPlotStrNodesInit, "GenericBlockPlotStr", "GenericPlotStr");

