/*
 *  opf.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef OPF_DOT_H
#define OPF_DOT_H
class ostream ;
ostream& OpenFile(const char * Name, const char * type) ;
void EmitHeaderIncludes(ostream& Str, int DfNodeFlag=0) ;
void EmitBodyIncludes(ostream& Str) ;

extern const char * ObjProGen ;
extern const char * ObjProDSP ;
extern const char * ObjProArith ;
extern const char * ObjProCom ;
extern const char * ObjProGui ;
extern const char * ObjProUsr ;
extern const char * ObjProIntUsr ;
extern const char * ObjProTarUsr ;
extern const char * ObjProNet;
extern const char * OpdRoot ;

#endif /* #ifdef OPF_DOT_H */
