/*
 *  dsppp_b.tab.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
typedef union {
	int32 Int ;
	double Float ;
	const char * Name ;
	struct ValueType * Value ;
	} YYSTYPE;
#define	IF	258
#define	ELSE	259
#define	NET_EDIT_COMMAND	260
#define	NAME	261
#define	STRING	262
#define	INTEGER	263
#define	MENU	264
#define	STATIC	265
#define	FLOATING	266
#define	DECL	267
#define	DECL_NAME	268
#define	PLUS_EQUAL	269
#define	MINUS_EQUAL	270
#define	OR	271
#define	AND	272
#define	EQU	273
#define	NEQ	274
#define	LEQ	275
#define	GEQ	276
#define	SHIFT_R	277
#define	SHIFT_L	278
#define	ARROW	279


extern YYSTYPE yylval;
