/*
 * This following header is included in every main program and in some program
 * in every library so the text of this copyright notice is in every 
 * distributed executable and library. Please ADD your own
 * copyright notice to programs that include code you hold the
 * copyright to.
 *
 * This is a dummy program to force inclusion of the copyright notice in the
 * library. It will not be included in the executable as a result of
 * linking with this library. If your main program uses ObjectProDSP
 * code but is not not generated by ObjectProDSP you should include the
 * header manually in any executables you distribute.
 */



#include "ObjProGen/cpyrght_exe.h" 
