/*
 *  rofftex.tab.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
typedef union {
	const char * Name ;
	int Int ;
	const char * String ;
	int Terminal ;
	enum TableOptionSelect TabOptSel ;
	} YYSTYPE;
#define	QUOTED_STRING	258
#define	TEXT_LINE	259
#define	TABLE_FORMAT	260
#define	TABLE_ENTRY	261
#define	TS_OPTION_PARAMETER	262
#define	VERBATIM_LINE	263
#define	CAPTION_STRING	264
#define	VERBATIM_REPLACE_LINE	265
#define	VERBATIM_LINE_SKIP	266
#define	ROFF_H	267
#define	ROFF_AL	268
#define	ROFF_ALA	269
#define	ROFF_LI	270
#define	ROFF_TS	271
#define	ROFF_TE	272
#define	ROFF_P	273
#define	ROFF_NF	274
#define	ROFF_FI	275
#define	EMPTY_VERBATIM	276
#define	END_OF_TABLE_LINE	277
#define	ROFF_LE	278
#define	TAB	279
#define	INTEGER_VAL	280
#define	CAPTION	281
#define	TABLE_OPTIONS	282
#define	SHIFT_R	283
#define	SHIFT_L	284
#define	LEQ	285
#define	GEQ	286
#define	EQU	287
#define	NEQ	288
#define	AND	289
#define	OR	290


extern YYSTYPE yylval;
