/*  tarparm.C   */
/*  Copyright 1991 Mountain Math Software  */
/*  All Rights Reserved                    */
#include "tarparm.h"

const char * InputLinksSuffix = "xXx_InputLink" ;
const char * OutputLinksSuffix = "xXx_OutputLink" ;
const char * ExecutionSequenceSuffix = "xXx_ExecSeq" ;
const char * generic_cpp_name = "generic_cpp" ;
const char * TIC30_name  = "TIC30" ;

