/*
 *  prsdrv.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  prsdrv.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */

#include "dectype.h"

class Stack ;

struct ValueType ;
extern ValueType * AlreadyDeclared(ValueType * Obj);
extern ValueType * MakeCopy(ValueType * Obj);
extern ValueType * SetArraySize(ValueType * , ValueType *);
extern struct ValueType * SelectArrayElement(ValueType * , ValueType *);
extern void IfPush(ValueType *Expression);
extern void Else();
extern void IfPop();
extern void IfPartEnd();
extern void NotConditional();
void IllegalChar(char x) ;
struct ValueType * DoProcedure(struct ValueType*Proc,struct ValueType*Param);
struct ValueType * MemberRef(struct ValueType* Obj, struct ValueType* Member);

extern Stack TheConditionalStack ;
int NoExecute() ;
int check_overflow(DecType type, double val);

