#!/bin/sh
#  make_both_tail.sh from ObjectProDSP 0.2
#  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
#  Licensed for free use and distribution under version 2 of the Gnu General
#  Public License. Please see file COPYING for details and restrictions.
#  
#  ObjectProDSP is a trademark of Mountain Math Software.
#  
. ../scripts/is_root_set.sh
cd $OPD_ROOT/build 
echo Building in directory "\``pwd`'."
INCLUDE_INT16=yes
export INCLUDE_INT16
make $* > make_output 2>& 1 &
tail +0f make_output
