/*
 *  call_back.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef CALL_BACK_DOT_H
#define CALL_BACK_DOT_H
#include "usriotyp.h"

class TxtWindow ;
class InputCallback {
	InputType type ;
public:
	InputCallback(InputType type);
	void record(const char * str);
	void WriteLine(const char *str, TxtWindow *);
	InputType get_type() const {return type;}
};
#endif /* #ifdef CALL_BACK_DOT_H */
