/*  firc.c   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#include "gcd.h"
#include "firc.h"

int DeltaIn(int16 Resample, int16 ZeroPad)
{
	if (!ZeroPad) return Resample ;
	return Resample / GreatestCommonDivisor(Resample,(ZeroPad+1));
}


int DeltaOut(int16 Resample,int16 ZeroPad)
{
	if (!ZeroPad) return 1 ;
	return (ZeroPad+1) / GreatestCommonDivisor(Resample,(ZeroPad+1));
}

