/*  menuprsl.c   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
#include <string.h>
#include <stream.h>
#include "menuprsl.h"
#include "mkstr.h"

void GetEmbeddedName(const char * yytext)
{
	int length = strlen(yytext);
	char * temp = new char[length-1] ;
	strncpy(temp,yytext+1,length-2);
	temp[length-2] = '\0' ;
	AddEmbeddedName(temp);
	delete temp ;
}

char * MakeOperatorCommand(const char * Base,const char * OperatorString)
{
	return Concatenate(Base," operator",OperatorString);
}
