#!/bin/sh
#  mkhlp.sh from ObjectProDSP 0.2
#  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
#  Licensed for free use and distribution under version 2 of the Gnu General
#  Public License. Please see file COPYING for details and restrictions.
#  
#  ObjectProDSP is a trademark of Mountain Math Software.
#  
#
Dest=$1
j=${2%.*}
ispell -l < $2 > $j.err
#diction $i >> $j.err
$OPD_ROOT/bin/cp_update_legal -c $Dest
( echo ".hys 100" ; echo ".nf" ; echo ".na" ;grep -v '\\\"\.' $j.roff) |
	ad_bp | nroff -mm  -Tascii  -rN4 | uniq  \
	| flt | sed -e 's/^          //' >> $Dest
cat $j.err
