#include "fstinit.h"
#include "incinit.h"
#include "meninit.h"

#include "hrdarth.h"

void InitAllMenuRoutines()
{
	AddDef = new Add("AddDef",
		2,
		0,
		1.0);	AsciiFileDef = new AsciiFile("AsciiFileDef",
		0,
		0,
		0,
		0);	BlockDef = new Block("BlockDef",
		2,
		1,
		0);	CircBufDesDef = new CircBufDes("CircBufDesDef",
		2048,
		512,
		0,
		1,
		4096,
		32);	CompareDiskDef = new CompareDisk("CompareDiskDef",
		0,
		1000,
		0.0,
		0);	ConstantDataDef = new ConstantData("ConstantDataDef",
		1024);	CosDef = new Cos("CosDef",
		0.125663706144,
		0.0,
		1024.);	CxCosDef = new CxCos("CxCosDef",
		0.125663706144,
		0.0,
		1024.);	CxFFTDef = new CxFFT("CxFFTDef",
		4,
		0.0,
		0.5,
		0);	CxFirDef = new CxFir("CxFirDef",
		1,
		0,
		0.0,
		0,
		(MachWord * ) TheDataForCxFirCoeffDefaultArray, 16);	CxImpDef = new CxImp("CxImpDef",
		32,
		0.0,
		1024.,
		.5,
		0);	NetworkDef = new Network("NetworkDef");	DataFlowDef = new DataFlow("DataFlowDef",
		* NetworkDef);	DemodDef = new Demod("DemodDef",
		0,
		0.0);	DemuxDef = new Demux("DemuxDef",
		2,
		1,
		1,
		1);	EyePlotDef = new EyePlot("EyePlotDef",
		400.,
		0);	FindStartTailDef = new FindStartTail("FindStartTailDef",
		0,
		0,
		0,
		0);	GainDef = new Gain("GainDef",
		1.0);	GainPadDef = new GainPad("GainPadDef",
		1.0,
		1,
		0);	HexListDef = new HexList("HexListDef",
		1,
		0);	ImportDataDef = new ImportData("ImportDataDef",
		0,
		"%d",
		1,
		0,
		(int32 * ) TheDataForImportDataSkipFieldsDefaultArray, 1,
		(int32 * ) TheDataForImportDataSkipColumnsDefaultArray, 1);	InputNodeDef = new InputNode("InputNodeDef",
		0,
		0,
		1);	InputWordDef = new InputWord("InputWordDef",
		0,
		0,
		0,
		0,
		1,
		1);	IntegrateDef = new Integrate("IntegrateDef",
		1,
		1,
		1.0);	InterpolateDef = new Interpolate("InterpolateDef",
		1,
		1);	ListingDef = new Listing("ListingDef",
		0,
		0);	MaskWordDef = new MaskWord("MaskWordDef",
		0);	MuxDef = new Mux("MuxDef",
		2,
		1,
		2,
		1);	NormalDef = new Normal("NormalDef",
		1024.0,
		0.0,
		1,
		1);	OutputNodeDef = new OutputNode("OutputNodeDef",
		0,
		1,
		1,
		128,
		0,
		1);	OutputWordDef = new OutputWord("OutputWordDef",
		0,
		0);	PackWordDef = new PackWord("PackWordDef",
		8,
		2);	PlotDef = new Plot("PlotDef",
		0);	PowerDef = new Power("PowerDef",
		0,
		1.0);	RampDef = new Ramp("RampDef",
		0,
		0,
		1);	ReadFloatDef = new ReadFloat("ReadFloatDef",
		0);	ReadIntDef = new ReadInt("ReadIntDef",
		0,
		0);	RealFirDef = new RealFir("RealFirDef",
		1,
		0,
		0,
		(MachWord * ) TheDataForRealFirCoeffDefaultArray, 16);	RepackStreamDef = new RepackStream("RepackStreamDef",
		8,
		1,
		0);	SampleDelayDef = new SampleDelay("SampleDelayDef",
		1,
		0);	TruncateDef = new Truncate("TruncateDef",
		31,
		31,
		0,
		0);	UniformNoiseDef = new UniformNoise("UniformNoiseDef",
		1024.0,
		-1024.0,
		1,
		1);	UnpackWordDef = new UnpackWord("UnpackWordDef",
		8,
		2,
		0);	VoiceNodeDef = new VoiceNode("VoiceNodeDef",
		0,
		0);	VoiceStripOutDef = new VoiceStripOut("VoiceStripOutDef",
		0);	ZFde0Def = new ZFde0("ZFde0Def",
		0,
		1000,
		.25);
}

