#include "getinf.h"
#include "cgidbg.h"
int GetTheInputFile(TargetNode * node, const char * FileName,
	const char * NodeName, int16 Flags, InputFile** FilePtr)
{
	// if (FileName)LogOut<<"GetTheInputFile, FileName = `"<<FileName<<"'\n" ;
	*FilePtr  = new InputFile(node,FileName,NodeName,Flags);
	
	if ((*FilePtr)->GetTheFile() < 1) {
		delete *FilePtr ;
		*FilePtr = 0 ;
		return 0 ;
	}
	return ((*FilePtr)->GetChannels());
}

