#include <stream.h>
#include <stdio.h>
#include "ObjProGen/cpyrght_exe.h"

main()
{
	const buf_size = 4097;
	char buf[buf_size];
	while(fgets(buf,buf_size,stdin)) {
		fputs(buf,stdout);
		if (*buf == '\n') fputs(".bp\n",stdout);
	}
}

