/*
 *  dsp.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  dsp.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
class DspCurrentState {
	int ParseRead ;
	int ExecutingMenuCommands;
	int DspProcessId ;
	int GuiProcessId ;
public:
	DspCurrentState(int dsp, int gui) {DspProcessId=dsp;
		GuiProcessId = gui; ParseRead=ExecutingMenuCommands=0;}
	int IsBusy() { return ExecutingMenuCommands || !ParseRead;}
	void MenuCommandStart();
	void MenuCommandComplete();
	void InformationComplete();
	void ActiveParseRead() {ParseRead = 1;}
	void ClearActiveParseRead() {ParseRead = 0 ;}
};

extern DspCurrentState * DspExecState ;

void ExitCleanUp() ;

