/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /home/daffy/u0/vern/flex/flex-2.4.7/RCS/flex.skl,v 1.2 94/08/03 11:13:24 vern Exp $
 */

#define FLEX_SCANNER

#include <stdio.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>
#include <unistd.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#ifdef __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */


#ifdef __TURBOC__
#define YY_USE_CONST
#endif


#ifndef YY_USE_CONST
#ifndef const
#define const
#endif
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.
 */
#define YY_START ((yy_start - 1) / 2)

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file".  Now included
 * only for backward compatibility with previous versions of flex.
 */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#ifdef __cplusplus
extern "C" {
#endif
	extern int yywrap YY_PROTO(( void ));
#ifdef __cplusplus
	}
#endif

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

static void yyunput YY_PROTO(( int c, char *buf_ptr ));
void yyrestart YY_PROTO(( FILE *input_file ));
void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void yy_load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));

static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
static void yy_push_state YY_PROTO(( int new_state ));
static void yy_pop_state YY_PROTO(( void ));
static int yy_top_state YY_PROTO(( void ));

static void *yy_flex_alloc YY_PROTO(( unsigned int ));
static void *yy_flex_realloc YY_PROTO(( void *, unsigned int ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define INITIAL 0
#define TEXT_MODE 1
#define TS_OPTIONS 2
#define TS_OPTIONS_PARAM 3
#define TS_FORMAT 4
#define TS_DATA 5
#define CMD_MODE 6
#define VERB 7
#define VERB_ONE 8
typedef unsigned char YY_CHAR;
typedef int yy_state_type;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
extern char *yytext;
#define yytext_ptr yytext

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, const char *, int ));
#endif

#ifdef __cplusplus
static int yyinput YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif

static yy_state_type yy_get_previous_state YY_PROTO(( void ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state ));
static int yy_get_next_buffer YY_PROTO(( void ));
static void yy_fatal_error YY_PROTO(( const char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = yy_cp - yy_bp; \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_END_OF_BUFFER 73
static const short int yy_accept[329] =
    {   0,
        0,    0,    0,    0,   26,   26,   31,   31,    0,    0,
       36,   36,    0,    0,    0,    0,    0,    0,   73,   71,
       47,   55,   53,   51,   66,   65,   62,   50,   67,   48,
       57,   43,   44,   60,   68,   58,   49,   59,   61,   63,
       71,   64,   56,   69,   52,   70,   54,   71,   41,   41,
       41,   41,   41,   41,   41,   41,   41,   41,   41,   41,
       41,   41,   41,   41,   41,   41,   41,   41,   71,   41,
       41,   41,   41,   41,   41,   26,   28,   32,   27,   71,
       31,   55,   53,   51,   66,   65,   62,   50,   67,   48,
       57,   43,   44,   60,   32,   58,   49,   59,   61,   63,

       71,   64,   56,   69,   52,   70,   54,   71,   49,   71,
       52,   38,   35,   40,   36,   38,   38,   38,   38,   38,
       38,   38,   38,   38,   38,   38,   38,   38,   38,   38,
       38,   38,   38,   38,   71,   38,   38,   38,   38,   38,
       38,   38,   24,    8,   71,   71,   22,   55,   53,   51,
       66,   65,   62,   50,   67,   48,   57,   43,   44,   60,
       68,   58,   49,   59,   61,   63,   71,   64,   56,   69,
       52,   70,   54,   46,   43,   45,   44,    0,    0,    0,
       11,    0,    0,   10,    0,    0,    0,   41,   41,   41,
       41,   42,   26,   27,    0,   29,   31,   43,   45,   44,

        0,    0,    0,   11,    0,    0,   10,    0,    0,    0,
       33,    0,   37,    0,   36,   37,   37,   37,   39,    0,
       37,   37,   11,   37,   37,   10,   37,   37,   37,   24,
        9,    0,   25,    0,   23,   43,   45,   44,    0,   45,
        0,    6,    0,   13,   18,   15,   14,    0,   16,   21,
       20,   19,   41,   42,   30,   31,   30,   37,    0,   13,
       18,   15,   14,   37,   16,   34,   21,   20,   19,   45,
        0,    6,    0,    0,    7,    0,    0,    0,   12,    0,
       17,    0,   12,   37,    0,    0,    7,    0,    0,    0,
        0,    5,    0,    0,    0,    0,    5,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,   23,    0,    0,    2,
        0,    2,    0,    1,    4,    3,    1,    0
    } ;

static const int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    4,    5,    6,    1,    1,    7,    8,    1,    9,
       10,   11,   12,   13,   14,   15,   16,   17,   18,   18,
       18,   18,   18,   18,   18,   19,   19,   20,   21,   22,
       23,   24,   25,    1,   26,   27,   28,   27,   29,   30,
        1,   31,   32,    1,    1,   33,    1,   34,   35,   36,
        1,   37,   38,   39,    1,    1,    1,   40,    1,    1,
       41,   42,   43,   44,   37,    1,   45,   46,   45,   46,

       46,   47,   48,   48,   49,   48,   48,   50,   48,   51,
       48,   48,   48,   50,   50,   48,   48,   48,   48,   52,
       48,   48,   53,   54,   55,   56,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const int yy_meta[57] =
    {   0,
        1,    2,    3,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    4,    1,    5,    5,    5,    1,
        1,    1,    4,    1,    1,    6,    5,    6,    5,    5,
        1,    1,    4,    4,    1,    1,    4,    4,    1,    1,
        1,    1,    1,    1,    6,    5,    5,    1,    1,    4,
        4,    1,    1,    4,    1,    1
    } ;

static const short int yy_base[351] =
    {   0,
        0,  896,    0,   56,  109,  158,  210,  895,  244,  284,
      338,  894,  113,  118,    0,  394,    0,  893,  907,  912,
      903,  912,  912,  912,  912,  912,  912,  912,  912,  912,
      912,  111,  108,  912,  912,  912,  912,  912,  912,  912,
      899,  912,  912,  912,  912,  912,  912,  425,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,  124,
      117,    0,    0,    0,    0,    0,    0,    0,  898,    0,
        0,    0,    0,    0,    0,  899,  912,  912,   98,  892,
      127,  891,  890,  889,  888,  887,  886,  885,  884,  883,
      882,  273,  164,  881,  880,  879,  878,  877,  876,  875,

      159,  874,  873,  872,  871,  870,  869,  467,  863,  862,
      861,  110,  912,  872,  870,  128,  135,  136,  149,  264,
      267,  269,  272,  281,  282,  450,  284,  288,  289,  449,
      453,  462,  465,  466,  165,  467,  469,  470,  471,  474,
      475,  515,  182,  870,  866,  868,  867,  866,  865,  864,
      863,  862,  861,  860,  859,  858,  857,  507,  177,  856,
      780,  760,  754,  753,  750,  745,  169,  739,  733,  729,
      728,  727,  726,  912,  170,    0,  180,  186,  695,  695,
      912,  161,  696,  912,  164,  676,  677,    0,  257,  550,
      463,    0,  719,  481,  712,  711,  323,  287,  710,  525,

      305,  286,  452,  709,  505,  270,  707,  512,  450,  269,
      912,  701,  479,  481,  711,  556,  595,  566,  912,  549,
      549,  561,  516,  577,  559,  552,  588,  586,  568,  461,
      912,  708,  912,  710,  912,  615,  640,  628,  572,    0,
      709,  912,  620,  685,  912,  912,  912,  706,  912,  912,
      912,  912,    0,    0,  912,  704,  704,    0,  622,  630,
      596,  607,  632,  658,  635,  640,  647,  648,  650,    0,
      703,  912,  668,  700,  912,  696,  672,  665,  912,  685,
      912,  652,  660,  676,  691,  680,  912,  660,  535,  673,
      617,  912,  571,  571,  570,  599,  912,  552,  502,  547,

      467,  465,  449,  662,  457,  484,  483,  678,  334,  301,
      329,  325,  284,  281,  536,  266,  197,  151,  163,  912,
      135,  912,  670,  912,  912,  912,  120,  912,  733,  739,
      745,  748,  754,  760,  766,  768,  774,  780,  786,  792,
      798,  804,  810,  816,  822,  828,  834,  840,  846,  852
    } ;

static const short int yy_def[351] =
    {   0,
      328,    1,    1,  328,    1,    1,  328,    7,    1,    1,
      328,   11,    1,    1,    1,  328,    1,    1,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  329,  329,
      329,  329,  329,  329,  329,  329,  329,  329,  329,  329,
      329,  329,  329,  329,  329,  329,  329,  329,  330,  329,
      329,  329,  329,  329,  329,  328,  328,  328,  328,  331,
      331,  331,  331,  331,  331,  331,  331,  331,  331,  331,
      331,  331,  331,  331,  331,  331,  331,  331,  331,  331,

      331,  331,  331,  331,  331,  331,  331,  331,  332,  332,
      332,  333,  328,  328,  328,  333,  333,  333,  333,  333,
      333,  333,  333,  333,  333,  333,  333,  333,  333,  333,
      333,  333,  333,  333,  328,  333,  333,  333,  333,  333,
      333,  333,  328,  328,  334,  335,  328,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  328,  328,  336,  328,  337,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  329,  329,  329,
      329,  338,  328,  328,  328,  328,  328,  328,  336,  328,

      337,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  332,  333,  333,  328,  333,  333,  333,  328,  337,
      333,  333,  333,  333,  333,  333,  333,  333,  333,  328,
      328,  334,  328,  335,  328,  335,  335,  335,  339,  336,
      340,  328,  328,  328,  328,  328,  328,  341,  328,  328,
      328,  328,  190,  338,  328,  328,  340,  217,  328,  333,
      333,  333,  333,  342,  333,  333,  333,  333,  333,  237,
      343,  328,  335,  340,  328,  344,  328,  328,  328,  341,
      328,  328,  333,  342,  342,  343,  328,  345,  335,  335,
      344,  328,  328,  328,  328,  345,  328,  335,  335,  328,

      328,  328,  335,  335,  328,  346,  347,  335,  348,  328,
      346,  349,  347,  350,  335,  348,  349,  328,  349,  328,
      350,  328,  335,  328,  328,  328,  335,    0,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328
    } ;

static const short int yy_nxt[969] =
    {   0,
       20,   20,   21,   20,   22,   20,   23,   24,   25,   26,
       27,   28,   29,   30,   20,   31,   32,   33,   33,   34,
       35,   36,   37,   38,   39,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       40,   41,   42,   43,   20,   20,   20,   20,   20,   20,
       20,   20,   44,   45,   46,   47,   49,   49,   21,   49,
       50,   49,   51,   52,   53,   54,   55,   56,   57,   58,
       48,   59,   60,   61,   61,   62,   63,   64,   65,   66,
       67,   49,   49,   49,   49,   49,   49,   49,   49,   49,
       49,   49,   49,   49,   49,   49,   68,   69,   70,   71,

       49,   49,   49,   49,   49,   49,   49,   49,   72,   73,
       74,   75,   76,  214,  143,  144,  143,   77,  145,  143,
      144,  143,  235,  145,  177,  177,  177,  175,  175,   78,
      197,  214,   48,  191,  191,  191,  196,  326,  214,  214,
      189,  189,  194,  194,  194,  194,  194,  194,  194,  194,
      176,  214,  214,   79,   79,   79,   79,   79,   79,   79,
       79,   76,  176,  190,  201,  325,   77,  219,  196,  214,
      220,  235,   48,  196,  239,  190,  214,  214,   78,  235,
      200,  200,  200,  230,  324,  230,  175,  175,  242,  246,
      214,  249,  247,  238,  238,  238,  177,  177,  177,  320,

      243,  250,   79,   79,   79,   79,   79,   79,   79,   79,
       80,   80,   21,   81,   82,   80,   83,   84,   85,   86,
       87,   88,   89,   90,   80,   91,   92,   93,   93,   94,
       95,   96,   97,   98,   99,   80,   80,   80,   80,   80,
       80,   80,   80,   80,   80,   80,   80,   80,   80,   80,
      100,  101,  102,  103,   80,   80,   80,   80,   80,   80,
       80,   80,  104,  105,  106,  107,  109,  214,  317,  110,
      214,  110,  214,  189,  189,  214,  110,  110,  255,  255,
      110,  110,  196,  322,  214,  214,  314,  214,  110,  198,
      198,  214,  214,  110,  110,  255,  255,  111,   48,  248,

      218,  218,  218,  175,  175,  214,  109,  242,  214,  110,
      214,  110,  199,  214,  257,  252,  110,  110,  244,  243,
      110,  110,  214,  214,  199,  214,  256,  320,  110,  214,
      214,  312,  255,  110,  110,  318,  317,  111,  112,  113,
      114,  115,  116,  112,  117,  118,  119,  120,  121,  122,
      123,  124,  112,  125,  126,  127,  127,  128,  129,  130,
      131,  132,  133,  112,  112,  112,  112,  112,  112,  112,
      112,  112,  112,  112,  112,  112,  112,  112,  134,  135,
      136,  137,  112,  112,  112,  112,  112,  112,  112,  112,
      138,  139,  140,  141,  146,  146,  147,  146,  148,  146,

      149,  150,  151,  152,  153,  154,  155,  156,   48,  157,
      158,  159,  159,  160,  161,  162,  163,  164,  165,  146,
      146,  146,  146,  146,  146,  146,  146,  146,  146,  146,
      146,  146,  146,  146,  166,  167,  168,  169,  146,  146,
      146,  146,  146,  146,  146,  146,  170,  171,  172,  173,
      179,  235,  214,  214,  180,  181,  214,  182,  183,  255,
      184,  255,  230,  185,  230,  214,  216,  216,  214,  214,
      214,  186,  214,  214,  214,  187,  196,  214,  214,  191,
      191,  191,  214,  245,  214,  314,  312,  308,  310,  217,
      214,  214,  202,  307,  214,  306,  203,  204,  251,  205,

      206,  217,  207,  214,  235,  208,  214,  214,  214,  235,
      214,  214,  214,  209,  255,  214,  214,  210,  214,  214,
      214,  255,  214,  236,  236,  194,  194,  194,  194,  194,
      194,  194,  194,  246,  255,  304,  247,  235,  235,  249,
      221,  177,  177,  177,  222,  223,  237,  224,  225,  250,
      226,  242,  214,  227,  235,  214,  214,  214,  237,  214,
      298,  228,  214,  259,  214,  229,  253,  253,  253,  214,
      323,  214,  216,  216,  272,  253,  253,  253,  253,  253,
      214,  260,  218,  218,  218,  305,  273,  303,  264,  214,
      214,  214,  261,  214,  253,  253,  253,  214,  214,  214,

      214,  297,  214,  302,  301,  262,  300,  214,  263,  214,
      214,  258,  258,  258,  269,  265,  266,  235,  214,  292,
      258,  258,  258,  258,  258,  267,  276,  214,  276,  214,
      235,  236,  236,  214,  268,  214,  214,  214,  214,  258,
      258,  258,  235,  214,  238,  238,  238,  277,  214,  277,
      214,  214,  278,  214,  282,  283,  270,  270,  270,  280,
      281,  285,  297,  214,  235,  270,  270,  270,  270,  270,
      235,  214,  235,  214,  288,  235,  214,  280,  281,  285,
      235,  214,  287,  295,  270,  270,  270,  281,  214,  214,
      309,  214,  280,  281,  285,  289,  294,  293,  292,  285,

      290,  214,  275,  327,  299,  287,  275,  256,  281,  315,
      279,  275,  235,  233,  215,  211,  255,  285,  255,  255,
      255,  255,  193,  252,  251,  248,  245,  244,  235,  235,
      235,  235,  285,  188,  188,  235,  188,  188,  188,  192,
      192,  235,  192,  192,  192,  195,  195,  235,  195,  195,
      195,  212,  235,  212,  213,  235,  235,  213,  213,  213,
      232,  232,  235,  232,  232,  232,  234,  234,  234,  234,
      234,  234,  240,  240,  241,  241,  241,  241,  241,  241,
      254,  254,  235,  254,  254,  254,  271,  271,  271,  271,
      271,  271,  274,  274,  274,  274,  274,  274,  280,  280,

      280,  280,  280,  280,  284,  284,  284,  284,  284,  284,
      286,  286,  286,  286,  286,  286,  291,  291,  291,  291,
      291,  291,  296,  296,  296,  296,  296,  296,  311,  311,
      311,  311,  311,  311,  313,  313,  313,  313,  313,  313,
      316,  316,  316,  316,  316,  316,  319,  319,  319,  319,
      319,  319,  321,  321,  321,  321,  321,  321,  235,  235,
      235,  235,  235,  235,  235,  235,  235,  235,  235,  174,
      235,  233,  231,  215,  174,  211,  211,  211,  196,  196,
      196,  196,  196,  196,  196,  196,  196,  196,  196,  196,
      196,  196,  196,  196,  196,  196,  196,  196,  196,  196,

      196,  196,  193,  178,  178,  174,  328,   48,  142,  108,
       48,   19,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328
    } ;

static const short int yy_chk[969] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,

        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    5,  112,   13,   13,   13,    5,   13,   14,
       14,   14,  327,   14,   33,   33,   33,   32,   32,    5,
       81,  116,   14,   61,   61,   61,   81,  321,  117,  118,
       60,   60,   79,   79,   79,   79,   79,   79,   79,   79,
       32,  112,  119,    5,    5,    5,    5,    5,    5,    5,
        5,    6,   32,   60,  101,  319,    6,  135,  101,  116,
      135,  167,    6,   93,  167,   60,  117,  118,    6,  159,
       93,   93,   93,  143,  318,  143,  175,  175,  178,  182,
      119,  185,  182,  159,  159,  159,  177,  177,  177,  317,

      178,  185,    6,    6,    6,    6,    6,    6,    6,    6,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    9,  120,  316,    9,
      121,    9,  122,  189,  189,  123,    9,    9,  210,  206,
        9,    9,   92,  314,  124,  125,  313,  127,    9,   92,
       92,  128,  129,    9,    9,  202,  198,    9,   10,  206,

      127,  127,  127,  198,  198,  120,   10,  201,  121,   10,
      122,   10,   92,  123,  201,  210,   10,   10,  202,  201,
       10,   10,  124,  125,   92,  127,  197,  312,   10,  128,
      129,  311,  197,   10,   10,  310,  309,   10,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   16,   16,   16,   16,   16,   16,

       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       48,  303,  130,  126,   48,   48,  131,   48,   48,  209,
       48,  203,  230,   48,  230,  132,  126,  126,  133,  134,
      136,   48,  137,  138,  139,   48,  108,  140,  141,  191,
      191,  191,  213,  203,  214,  307,  306,  303,  305,  126,
      130,  126,  108,  302,  131,  301,  108,  108,  209,  108,

      108,  126,  108,  132,  299,  108,  133,  134,  136,  158,
      137,  138,  139,  108,  205,  140,  141,  108,  142,  223,
      213,  208,  214,  158,  158,  194,  194,  194,  194,  194,
      194,  194,  194,  205,  200,  299,  205,  289,  315,  208,
      142,  200,  200,  200,  142,  142,  158,  142,  142,  208,
      142,  220,  221,  142,  298,  226,  142,  223,  158,  216,
      289,  142,  225,  220,  222,  142,  190,  190,  190,  218,
      315,  229,  216,  216,  239,  190,  190,  190,  190,  190,
      224,  221,  218,  218,  218,  300,  239,  298,  225,  228,
      221,  227,  222,  226,  190,  190,  190,  216,  217,  261,

      225,  296,  222,  295,  294,  224,  293,  218,  224,  229,
      262,  217,  217,  217,  229,  227,  227,  236,  224,  291,
      217,  217,  217,  217,  217,  227,  243,  228,  259,  227,
      238,  236,  236,  260,  228,  263,  217,  261,  265,  217,
      217,  217,  237,  266,  238,  238,  238,  243,  262,  259,
      267,  268,  243,  269,  259,  260,  237,  237,  237,  264,
      264,  264,  288,  283,  304,  237,  237,  237,  237,  237,
      273,  260,  323,  263,  273,  290,  265,  284,  284,  284,
      308,  266,  286,  282,  237,  237,  237,  280,  267,  268,
      304,  269,  285,  285,  285,  273,  278,  277,  276,  264,

      273,  283,  274,  323,  290,  271,  257,  256,  248,  308,
      244,  241,  234,  232,  215,  212,  207,  284,  204,  199,
      196,  195,  193,  187,  186,  183,  180,  179,  173,  172,
      171,  170,  285,  329,  329,  169,  329,  329,  329,  330,
      330,  168,  330,  330,  330,  331,  331,  166,  331,  331,
      331,  332,  165,  332,  333,  164,  163,  333,  333,  333,
      334,  334,  162,  334,  334,  334,  335,  335,  335,  335,
      335,  335,  336,  336,  337,  337,  337,  337,  337,  337,
      338,  338,  161,  338,  338,  338,  339,  339,  339,  339,
      339,  339,  340,  340,  340,  340,  340,  340,  341,  341,

      341,  341,  341,  341,  342,  342,  342,  342,  342,  342,
      343,  343,  343,  343,  343,  343,  344,  344,  344,  344,
      344,  344,  345,  345,  345,  345,  345,  345,  346,  346,
      346,  346,  346,  346,  347,  347,  347,  347,  347,  347,
      348,  348,  348,  348,  348,  348,  349,  349,  349,  349,
      349,  349,  350,  350,  350,  350,  350,  350,  160,  157,
      156,  155,  154,  153,  152,  151,  150,  149,  148,  147,
      146,  145,  144,  115,  114,  111,  110,  109,  107,  106,
      105,  104,  103,  102,  100,   99,   98,   97,   96,   95,
       94,   91,   90,   89,   88,   87,   86,   85,   84,   83,

       82,   80,   76,   69,   41,   21,   19,   18,   12,    8,
        2,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328,  328,  328,
      328,  328,  328,  328,  328,  328,  328,  328
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
char *yytext;
# line 1 "../rofftex.l"
/*
 *  rofftex.l from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software. All rights reserved.
 *  
 *  This file is part of ObjectProDSP, a tool for Digital Signal
 *  Processing design, development and implementation. It is free
 *  software provided you use and distribute it under the terms of
 *  version 2 of the GNU General Public License as published
 *  by the Free Software Foundation. You may NOT distribute it or
 *  works derived from it or code that it generates under ANY
 *  OTHER terms.  In particular NONE of the ObjectProDSP system is
 *  licensed for use under the GNU General Public LIBRARY License.
 *  Mountain Math Software plans to offer a commercial version of
 *  ObjectProDSP for a fee. That version will allow redistribution
 *  of generated code under standard commercial terms.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of version 2 of the GNU General
 *  Public License along with this program. See file COPYING. If not
 *  or if you wish information on commercial versions and licensing
 *  write Mountain Math Software, P. O. Box 2124, Saratoga, CA 95070,
 *  USA, or send us e-mail at: support@mtnmath.com.
 *  
 *  You may also obtain the GNU General Public License by writing the
 *  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *  USA.  However if you received a copy of this program without the
 *  file COPYING or without a copyright notice attached to all text
 *  files, libraries and executables please inform Mountain Math Software.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/* add code here with TAB at start of each line */
# line 44 "../rofftex.l"
#undef yywrap
#include "rofftex.h"
#include "rofftex.tab.h"
#include "lexmac.h"
#include "mkstr.h"

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = getc( yyin ); \
		result = c == EOF ? 0 : 1; \
		buf[0] = (char) c; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yylex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;

# line 50 "../rofftex.l"


	if ( yy_init )
		{
#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( yy_current_buffer )
			yy_init_buffer( yy_current_buffer, yyin );
		else
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();

		yy_init = 0;
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
		if ( yy_bp[-1] == '\n' )
			++yy_current_state;
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 329 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 912 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_USER_ACTION
# line 51 "../rofftex.l"
{
					BEGIN CMD_MODE;
					CaptionOn=1;
					ClearCaptionIndexTable();
					return CAPTION;
				}
	YY_BREAK
case 2:
YY_USER_ACTION
# line 57 "../rofftex.l"
{
					LineNumber+=2 ;
					yylval.String = make_verbatim_line(yytext);
					return VERBATIM_REPLACE_LINE ;
				}
	YY_BREAK
case 3:
YY_USER_ACTION
# line 62 "../rofftex.l"
{
					LineNumber+=2 ;
					yylval.String = make_verbatim_line(yytext);
                    return TABLE_ENTRY ;
				}
	YY_BREAK
case 4:
YY_USER_ACTION
# line 67 "../rofftex.l"
{
					LineNumber+=2 ;
					yylval.String = make_verbatim_line(yytext);
					return VERBATIM_LINE_SKIP ;
				}
	YY_BREAK
case 5:
YY_USER_ACTION
# line 72 "../rofftex.l"
{
					LineNumber++ ;
					yylval.String = Concatenate(yytext+2);
					return TEXT_LINE ;
				}
	YY_BREAK
case 6:
YY_USER_ACTION
# line 77 "../rofftex.l"
{ LineNumber++ ; Dbg cerr<< "Comment `"<<yytext<<"'\n"; }
	YY_BREAK
case 7:
YY_USER_ACTION
# line 78 "../rofftex.l"
{ LineNumber+=2 ;	Dbg cerr<< "Comment `"<<yytext<<"'\n"; }
	YY_BREAK
case 8:
YY_USER_ACTION
# line 79 "../rofftex.l"
{BEGIN TEXT_MODE; CaptionOn = 0; LineNumber++;}
	YY_BREAK
case 9:
YY_USER_ACTION
# line 80 "../rofftex.l"
{BEGIN TEXT_MODE; CaptionOn = 0; LineNumber+=2;
							return ROFF_P ;}
	YY_BREAK
case 10:
YY_USER_ACTION
# line 82 "../rofftex.l"
{BEGIN CMD_MODE; return ROFF_P ;}
	YY_BREAK
case 11:
YY_USER_ACTION
# line 83 "../rofftex.l"
{BEGIN CMD_MODE; return ROFF_H ;}
	YY_BREAK
case 12:
YY_USER_ACTION
# line 84 "../rofftex.l"
{BEGIN CMD_MODE; return ROFF_AL ;}
	YY_BREAK
case 13:
YY_USER_ACTION
# line 85 "../rofftex.l"
{BEGIN CMD_MODE; return ROFF_AL ;}
	YY_BREAK
case 14:
YY_USER_ACTION
# line 86 "../rofftex.l"
{BEGIN CMD_MODE; return ROFF_LI ;}
	YY_BREAK
case 15:
YY_USER_ACTION
# line 87 "../rofftex.l"
{BEGIN CMD_MODE; return ROFF_LE ;}
	YY_BREAK
case 16:
YY_USER_ACTION
# line 88 "../rofftex.l"
BEGIN CMD_MODE ;
	YY_BREAK
case 17:
YY_USER_ACTION
# line 89 "../rofftex.l"
{BEGIN VERB; LineNumber++; return ROFF_NF ;}
	YY_BREAK
case 18:
YY_USER_ACTION
# line 90 "../rofftex.l"
{BEGIN CMD_MODE; return ROFF_FI ;}
	YY_BREAK
case 19:
YY_USER_ACTION
# line 91 "../rofftex.l"
;
	YY_BREAK
case 20:
YY_USER_ACTION
# line 92 "../rofftex.l"
;
	YY_BREAK
case 21:
YY_USER_ACTION
# line 93 "../rofftex.l"
{BEGIN TS_OPTIONS; return ROFF_TS ;}
	YY_BREAK
case 22:
YY_USER_ACTION
# line 94 "../rofftex.l"
{
					LineNumber++ ;
					return EMPTY_VERBATIM ;
				}
	YY_BREAK
case 23:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_USER_ACTION
# line 98 "../rofftex.l"
{
					yylval.String = MakeLitString(yytext);
					return VERBATIM_LINE ;
				}
	YY_BREAK
case 24:
YY_USER_ACTION
# line 102 "../rofftex.l"
;
	YY_BREAK
case 25:
YY_USER_ACTION
# line 103 "../rofftex.l"
{
					Dbg cerr << "Found QUOTED_STRING\n" ;
					const char * Ret = MakeString(yytext);
					if (!CaptionOn) Ret = TeXString(Ret);
					yylval.String = Ret ;
					if (CaptionOn) return CAPTION_STRING ;
					return QUOTED_STRING ;
				}
	YY_BREAK
case 26:
YY_USER_ACTION
# line 111 "../rofftex.l"
;
	YY_BREAK
case 27:
YY_USER_ACTION
# line 112 "../rofftex.l"
{
					Dbg cerr << "TABLE_OPTIONS\n" ;
					yylval.TabOptSel = ExpandOption(yytext);
					return TABLE_OPTIONS ;
				}
	YY_BREAK
case 28:
YY_USER_ACTION
# line 117 "../rofftex.l"
BEGIN TS_OPTIONS_PARAM ;
	YY_BREAK
case 29:
YY_USER_ACTION
# line 118 "../rofftex.l"
{
					Dbg cerr << "TS_OPTION_PARAMETER 1\n" ;
					char * Temp = new char[2] ;
					Temp[0] = yytext[0];
					Temp[1] = '\0' ;
					yylval.String = Temp ;
					return TS_OPTION_PARAMETER ;
				}
	YY_BREAK
case 30:
YY_USER_ACTION
# line 126 "../rofftex.l"
{
					Dbg cerr << "TS_OPTION_PARAMETER 2\n" ;
					yylval.String = MakeLitString(yytext);
					return TS_OPTION_PARAMETER ;
				}
	YY_BREAK
case 31:
YY_USER_ACTION
# line 131 "../rofftex.l"
;
	YY_BREAK
case 32:
YY_USER_ACTION
# line 132 "../rofftex.l"
{
					Dbg cerr << "BEGIN TS_FORMAT\n" ;
					BEGIN TS_FORMAT ;
				}
	YY_BREAK
case 33:
YY_USER_ACTION
# line 136 "../rofftex.l"
{
					Dbg cerr << "TABLE_FORMAT\n" ;
					BEGIN TS_DATA ;
					yylval.String = MakeLitString(yytext);
					return TABLE_FORMAT ;
				}
	YY_BREAK
case 34:
YY_USER_ACTION
# line 142 "../rofftex.l"
{
					Dbg cerr << "END_OF_TABLE: ROFF_TE\n" ;
					BEGIN TEXT_MODE ;
					return ROFF_TE ;
				}
	YY_BREAK
case 35:
YY_USER_ACTION
# line 147 "../rofftex.l"
return TAB ;
	YY_BREAK
case 36:
YY_USER_ACTION
# line 148 "../rofftex.l"
;
	YY_BREAK
case 37:
# line 150 "../rofftex.l"
case 38:
YY_USER_ACTION
# line 150 "../rofftex.l"
{
					Dbg cerr << "TABLE_ENTRY\n" ;
					Dbg cerr << "`" << yytext << "'\n" ;
					yylval.String = TeXString(yytext);	
					return TABLE_ENTRY ;
				}
	YY_BREAK
case 39:
YY_USER_ACTION
# line 156 "../rofftex.l"
{
					Dbg cerr << "Table continue\n" ;
					LineNumber++ ;
					/* continue line */		
				}
	YY_BREAK
case 40:
YY_USER_ACTION
# line 161 "../rofftex.l"
{
					Dbg cerr << "END_OF_TABLE_LINE\n" ;
					LineNumber++ ;
					return END_OF_TABLE_LINE ;
				}
	YY_BREAK
case 41:
# line 167 "../rofftex.l"
case 42:
YY_USER_ACTION
# line 167 "../rofftex.l"
{
					Dbg cerr << "Found TEXT_LINE\n" ;
					Dbg cerr << "`" << yytext << "'\n" ;
					yylval.String = TeXString(yytext);
					return TEXT_LINE ;
				}
	YY_BREAK
case 43:
YY_USER_ACTION
# line 173 "../rofftex.l"
{
					yylval.Int = OctConvert(yytext);
					return INTEGER_VAL ;
				}
	YY_BREAK
case 44:
YY_USER_ACTION
# line 177 "../rofftex.l"
{
					yylval.Int = IntConvert(yytext);
					return INTEGER_VAL ;
				}
	YY_BREAK
case 45:
YY_USER_ACTION
# line 181 "../rofftex.l"
{	
					yylval.Int = HexConvert(yytext+2);
					return INTEGER_VAL ;
				}
	YY_BREAK
case 46:
YY_USER_ACTION
# line 185 "../rofftex.l"
{
					return ROFF_P ;
					LineNumber+=2 ;
				}
	YY_BREAK
case 47:
YY_USER_ACTION
# line 189 "../rofftex.l"
{
					LineNumber++;
					Dbg cerr<<"Line "<< LineNumber << "\n";
				}
	YY_BREAK
case 48:
YY_USER_ACTION
# line 193 "../rofftex.l"
return yytext[0] ;
	YY_BREAK
case 49:
# line 195 "../rofftex.l"
case 50:
# line 196 "../rofftex.l"
case 51:
# line 197 "../rofftex.l"
case 52:
# line 198 "../rofftex.l"
case 53:
# line 199 "../rofftex.l"
case 54:
# line 200 "../rofftex.l"
case 55:
# line 201 "../rofftex.l"
case 56:
# line 202 "../rofftex.l"
case 57:
# line 203 "../rofftex.l"
case 58:
# line 204 "../rofftex.l"
case 59:
# line 205 "../rofftex.l"
case 60:
# line 206 "../rofftex.l"
case 61:
# line 207 "../rofftex.l"
case 62:
# line 208 "../rofftex.l"
case 63:
# line 209 "../rofftex.l"
case 64:
# line 210 "../rofftex.l"
case 65:
# line 211 "../rofftex.l"
case 66:
# line 212 "../rofftex.l"
case 67:
# line 213 "../rofftex.l"
case 68:
# line 214 "../rofftex.l"
case 69:
# line 215 "../rofftex.l"
case 70:
# line 216 "../rofftex.l"
case 71:
YY_USER_ACTION
# line 216 "../rofftex.l"
IllegalChar(yytext[0]);
	YY_BREAK
case 72:
YY_USER_ACTION
# line 217 "../rofftex.l"
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(TEXT_MODE):
case YY_STATE_EOF(TS_OPTIONS):
case YY_STATE_EOF(TS_OPTIONS_PARAM):
case YY_STATE_EOF(TS_FORMAT):
case YY_STATE_EOF(TS_DATA):
case YY_STATE_EOF(CMD_MODE):
case YY_STATE_EOF(VERB):
case YY_STATE_EOF(VERB_ONE):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = yy_cp - yytext_ptr - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
							yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr - 1; /* copy prev. char, too */
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a singled characater, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = yy_c_buf_p - yytext_ptr;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset = yy_c_buf_p - b->yy_ch_buf;

			b->yy_buf_size *= 2;
			b->yy_ch_buf = (char *)
				yy_flex_realloc( (void *) b->yy_ch_buf,
						 b->yy_buf_size );

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move - YY_MORE_ADJ == 1 )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	/* yytext begins at the second character in yy_ch_buf; the first
	 * character is the one which preceded it before reading in the latest
	 * buffer; it needs to be kept around in case it's a newline, so
	 * yy_get_previous_state() will have with '^' rules active.
	 */

	yytext_ptr = &yy_current_buffer->yy_ch_buf[1];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	register char *yy_bp = yytext_ptr;

	yy_current_state = yy_start;
	if ( yy_bp[-1] == '\n' )
		++yy_current_state;

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 329 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
#endif
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 329 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 328);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp )
#else
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
#endif
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += dest - source;
		yy_bp += dest - source;
		yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	if ( yy_cp > yy_bp && yy_cp[-1] == '\n' )
		yy_cp[-2] = '\n';

	*--yy_cp = (char) c;


	/* Note: the formal parameter *must* be called "yy_bp" for this
	 * macro to now work correctly.
	 */
	YY_DO_BEFORE_ACTION; /* set up yytext again */
	}


#ifdef __cplusplus
static int yyinput()
#else
static int input()
#endif
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			yytext_ptr = yy_c_buf_p;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						{
						yy_c_buf_p =
						yytext_ptr + YY_MORE_ADJ;
						return EOF;
						}

					YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;
					break;

				case EOB_ACT_LAST_MATCH:
#ifdef __cplusplus
					YY_FATAL_ERROR(
					"unexpected last match in yyinput()" );
#else
					YY_FATAL_ERROR(
					"unexpected last match in input()" );
#endif
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;

	return c;
	}


#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file )
#else
void yyrestart( input_file )
FILE *input_file;
#endif
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void yy_load_buffer_state( void )
#else
void yy_load_buffer_state()
#endif
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );

	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );

	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	yy_init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b )
#else
void yy_delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	yy_flex_free( (void *) b->yy_ch_buf );
	yy_flex_free( (void *) b );
	}


#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif
	{
	b->yy_input_file = file;

	/* We put in the '\n' and start reading from [1] so that an
	 * initial match-at-newline will be true.
	 */

	b->yy_ch_buf[0] = '\n';
	b->yy_n_chars = 1;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[2] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[1];

	b->yy_is_interactive = file ? isatty( fileno(file) ) : 0;

	b->yy_fill_buffer = 1;

	b->yy_buffer_status = YY_BUFFER_NEW;
	}


#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state )
#else
static void yy_push_state( new_state )
int new_state;
#endif
	{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
		{
		int new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yy_flex_realloc(
					(void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}


static void yy_pop_state()
	{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
	}


static int yy_top_state()
	{
	return yy_start_stack[yy_start_stack_ptr - 1];
	}


#ifdef YY_USE_PROTOS
static void yy_fatal_error( const char msg[] )
#else
static void yy_fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( 1 );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n - YY_MORE_ADJ; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, const char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
const char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( unsigned int size )
#else
static void *yy_flex_alloc( size )
unsigned int size;
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, unsigned int size )
#else
static void *yy_flex_realloc( ptr, size )
void *ptr;
unsigned int size;
#endif
	{
	return (void *) realloc( ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}
# line 217 "../rofftex.l"
