/*
 *  decptr.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  decptr.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */

#ifndef DECPTR_DOT_H
#define DECPTR_DOT_H
union DeclarationPtr {
	class SimpleUserObject *   Basic ;
	class InteractiveEntity *  Entity ;
	class InteractiveEntityList * EntityList ;

	SimpleUserObject * Set(SimpleUserObject * Obj)
		{return (Basic = Obj) ;}
	InteractiveEntity * Set(InteractiveEntity * IntEnt)
		{ return (Entity = IntEnt) ;}
	InteractiveEntityList * Set(InteractiveEntityList * IntEntList)
		{ return (EntityList = IntEntList) ;}
} ;

extern DeclarationPtr UnTheNodes ;
extern DeclarationPtr UnTheCmpdNodes ;
extern DeclarationPtr UnTheDisplays ;
extern DeclarationPtr UnTheSignals ;
extern DeclarationPtr UnTheNets ;
#endif /* #ifdef DECPTR_DOT_H */
