/*
 *  filename.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef FILENAME_DOT_H
#define FILENAME_DOT_H
/*  filename.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */

char * MakeFullName(const char * DirName,const char * Name,
	const char * SubDir=0) ;

#endif /* #ifdef FILENAME_DOT_H */
