#include <stream.h>
#include <iomanip.h>
#include "gen_data.h"

void GenData::out_val(int val, format f)
{
	if (total_count >= total_out) return ;
	switch(f) {
case f_dec:
		*str << setw(width) << val ;
		break ;
case f_hex:
		*str << "0x" << setw(width) << hex << val << dec  ;
		break ;
case f_oct:
		*str << "0" << oct << setw(width) << val << dec ;
		break ;
case f_none:
		break ;
	}
	field_out();
}

