/*
 *  y.tab.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  y.tab.h   */
/*  Copyright 1991 Mountain Math Software  */
/*  All Rights Reserved                    */

typedef union  {
	const char * Name ;
	int Int ;
	const char * String ;
	int Terminal ;
	enum TableOptionSelect TabOptSel ;
	} YYSTYPE;
extern YYSTYPE yylval;
# define QUOTED_STRING 257
# define TEXT_LINE 258
# define TABLE_FORMAT 259
# define TABLE_ENTRY 260
# define TS_OPTION_PARAMETER 261
# define VERBATIM_LINE 262
# define CAPTION_STRING 263
# define ROFF_H 264
# define ROFF_AL 265
# define ROFF_ALA 266
# define ROFF_LI 267
# define ROFF_TS 268
# define ROFF_TE 269
# define ROFF_P 270
# define ROFF_NF 271
# define ROFF_FI 272
# define EMPTY_VERBATIM 273
# define END_OF_TABLE_LINE 274
# define TAB 275
# define ROFF_LE 276
# define INTEGER_VAL 277
# define CAPTION 278
# define TABLE_REPLACE_LINE 279
# define TABLE_OPTIONS 280
# define SHIFT_R 281
# define SHIFT_L 282
# define LEQ 283
# define GEQ 284
# define EQU 285
# define NEQ 286
# define AND 287
# define OR 288
