/*
 *  mentyp.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  mentyp.h   */
/*  Copyright 1990 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef MENTYP_DOT_H
#define MENTYP_DOT_H

enum MenuType {MenuTypeInvalid,MenuTypeCommand,MenuTypeMenu,
		MenuTypeExtension, MenuTypeHelp};

enum MenuTypeSpecial {MenuTypeSpecialNormal=0,MenuTypeSpecialExpanded=1};

enum DoMenuReturn {DoMenuPrevious,DoMenuMain,DoMenuCurrent,DoMenuHelp} ;

class MenuEnums {
public:
    enum ExitStatus {not_found,found,abort};
};



#endif /* #ifdef MENTYP_DOT_H */
