/*
 *  menu_b.tab.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
typedef union {
	int32 Int ;
	const char * Name ;
	union Value * Val ;
	struct TextFragmentList * Text ;
	struct Double * Dbl ;
	class MenuType * TheMenuType ;
	struct Compound * Tree ;
	} YYSTYPE;
#define	HELP_DEF	258
#define	DEF_HELP_FILE	259
#define	CONFLICT	260
#define	WAIT	261
#define	DEFINED_MENU	262
#define	DEFINED_ACTION	263
#define	MENU_ADDITION	264
#define	ITEM_REF	265
#define	DEFAULT	266
#define	ITEM_HELP	267
#define	MENU_LOCAL	268
#define	MENU_REMOTE	269
#define	MENU_REMOTE_OPTIONS	270
#define	MAINS	271
#define	INIT	272
#define	INCLUDE_LOCAL	273
#define	INCLUDE_REMOTE	274
#define	MULTIPLE	275
#define	USE	276
#define	DYNAMIC	277
#define	HISTORY	278
#define	SELECT	279
#define	ADD_TO_MENU	280
#define	TEMPLATE	281
#define	ORPHAN	282
#define	SHIFT_R	283
#define	SHIFT_L	284
#define	LEQ	285
#define	GEQ	286
#define	EQU	287
#define	NEQ	288
#define	AND	289
#define	OR	290
#define	OPERATOR	291
#define	NAME	292
#define	STRING	293
#define	LOCAL	294
#define	REMOTE	295
#define	LOCAL_REMOTE	296
#define	REMOTE_OPTIONS	297
#define	MENU	298
#define	NEW_MENU	299
#define	HELP_FILE	300
#define	DYNAMIC_MENU	301
#define	TEXT	302
#define	INTEGER_VAL	303
#define	DOUBLE_VAL	304
#define	USER_COPYRIGHT	305
#define	EXPLICIT_STRING	306


extern YYSTYPE yylval;
