/*
 *  rofftex.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef ROFFTEX_DOT_H
#define ROFFTEX_DOT_H
#include <stream.h>
#include "totex.h"
// #include "yacoper.h"
#include "stdyac.h"
#include "lexnum.h"
#endif /* #ifdef ROFFTEX_DOT_H */
