#!/bin/sh
#  make_inputs_exc.sh from ObjectProDSP 0.2
#  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
#  Licensed for free use and distribution under version 2 of the Gnu General
#  Public License. Please see file COPYING for details and restrictions.
#  
#  ObjectProDSP is a trademark of Mountain Math Software.
#  
echo "cd $1 ;" 'rm *.nod'
OPD_FILES="$2/*.nod"
for i in $OPD_FILES 
do
	#echo i is $i
	OPD_DO=DO
	for j in $* 
	do
		#echo j is $j
		case $j in
$1 | $2) continue  ;;
		esac
	
		OPD_FILE=${i##*/}
		OPD_CHECK=${OPD_FILE%\.nod}
		#echo $OPD_CHECK
		if [ $j = $OPD_CHECK ]
		then
			#echo continue for $j
			continue 2
		fi
	done
	echo cp "$i ."
done
