/*
 *  genmenu.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  genmenu.h   */
/*  Copyright 1989 Mountain Math Software  */
/*  All Rights Reserved                    */
void WriteMenuData(/* const char * , const char * , */ const char * NodeName,
	class NodeParameterList *, struct Description *);

void EmitHelpLine(class MofStream * Stream, class TextFragmentList * Text) ;
void CloseMenuFile();
void CreateRemoteHeader(const char * RemoteName, const char * IncludeName);
void CreateLocalHeader(const char * LocalName) ;
extern char * MenuDirectory ;
void OpenNodFile(const char * NodeName, const char * HeaderFileName) ;
void EmitSelectMemberToExecute(const char * NodeName,
	const char * MenuType = "History") ;
void EmitDescribeMembersMenu(const char * NodeName,
	const char * MenuType= "History") ;
void WriteBaseClass(const char * NodeName, const char * Base) ;

extern int DefaultInstance ;

