/*
 *  menmgr.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
#ifndef MENMGR_DOT_H
#define MENMGR_DOT_H
#include "slist.h"

class MenuView ;
class Menu ;

class MenuViewList:public SingleList {
public:
    ErrCode Insert(MenuView *nt) {return SingleList::Insert(nt);}
    ErrCode Append(MenuView *nt) {return SingleList::Append(nt);}
    MenuView * Get()   {return (MenuView *) SingleList::Get();}
    MenuView * Pop() {return (MenuView *) SingleList::Pop();}
    MenuViewList(){;}
    int Size(){return SingleList::Size();}
	// ErrCode RemoveEntry(x) will remove x
	void about_to_change_menu(Menu * to_change);
} ;

class MenuViewListIterator:private SingleListIterator {
public:
    MenuViewListIterator(MenuViewList& df):SingleListIterator((SingleList&) df){}
    MenuView * operator()()
        {return (MenuView *) Next();}
};


#endif /* #ifdef MENMGR_DOT_H */
