/*
 *  includ.h from ObjectProDSP 0.2
 *  Copyright (C) 1994 1995, Mountain Math Software, All rights reserved.
 *  Licensed for free use and distribution under version 2 of the Gnu General
 *  Public License. Please see file COPYING for details and restrictions.
 *  
 *  ObjectProDSP is a trademark of Mountain Math Software.
 */
/*  includ.h   */
/*  Copyright 1991 Mountain Math Software  */
/*  All Rights Reserved                    */
#ifndef INCLUD_DOT_H
#define INCLUD_DOT_H
class MofStream  ;
extern void EmitIncludeFiles(MofStream& Header, MofStream& Cpp, int FirstFlag);
extern void InitTheIncludes() ;
const InteractiveFileOnly = 0 ;
const TargetFileOnly = 1  ;

#endif /* #ifdef INCLUD_DOT_H */
